// Emacs style mode select   -*- C++ -*-
// -----------------------------------------------------------------------------
// ########   ###### #####   #####  ######   ######  ######
// ##     ##  ##     ##  ## ##  ## ##    ## ##    ## ##   ##
// ##     ##  ##     ##   ###   ## ##    ## ##    ## ##    ##
// ########   ####   ##    #    ## ##    ## ##    ## ##    ##
// ##    ##   ##     ##         ## ##    ## ##    ## ##    ##
// ##     ##  ##     ##         ## ##    ## ##    ## ##   ##
// ##      ## ###### ##         ##  ######   ######  ######
//                      http://remood.sourceforge.net/
// -----------------------------------------------------------------------------
// Project Leader:    GhostlyDeath           (ghostlydeath@gmail.com)
// Project Co-Leader: RedZTag                (jostol27@gmail.com)
// Members:           Demyx                  (demyx@endgameftw.com)
// -----------------------------------------------------------------------------
// Copyright (C) 1993-1996 by id Software, Inc.
// Portions Copyright (C) 1998-2000 by DooM Legacy Team.
// Portions Copyright (C) 2008-2009 The ReMooD Team..
// -----------------------------------------------------------------------------
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// -----------------------------------------------------------------------------
// DESCRIPTION:
//      DOOM graphics stuff for SDL

#include <stdlib.h>

#include <SDL.h>

#include "doomdef.h"
#include "doomstat.h"
#include "i_system.h"
#include "v_video.h"
#include "m_argv.h"
#include "m_menu.h"
#include "d_main.h"
#include "s_sound.h"
#include "g_input.h"
#include "st_stuff.h"
#include "g_game.h"
#include "i_video.h"
#include "console.h"
#include "command.h"

#ifdef _WIN32
#include <windows.h>
#include "SDL_syswm.h"
#else
unsigned char ICON[4096] =
{
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x62, 0x00, 0x00, 0xFF, 0x53, 0x00, 0x00, 0xFF, 0x3B, 0x00, 0x00, 0xFF, 0x2A, 0x00, 0x00, 0xFF,
	0x1B, 0x00, 0x00, 0xFF, 0x1F, 0x00, 0x00, 0xFF, 0x48, 0x00, 0x00, 0xFF, 0x53, 0x00, 0x00, 0xFF,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x00, 0x00, 0x00, 0x00, 0x88, 0x00, 0x00, 0xFF, 0x90, 0x00, 0x00, 0xFF, 0x8E, 0x00, 0x00, 0xFF,
	0x84, 0x00, 0x00, 0xFF, 0x75, 0x00, 0x00, 0xFF, 0x52, 0x00, 0x00, 0xFF, 0x45, 0x00, 0x00, 0xFF,
	0x2D, 0x00, 0x00, 0xFF, 0x1B, 0x00, 0x00, 0xFF, 0x39, 0x00, 0x00, 0xFF, 0x34, 0x00, 0x00, 0xFF,
	0x30, 0x00, 0x00, 0xFF, 0x21, 0x00, 0x00, 0xFF, 0x23, 0x00, 0x00, 0xFF, 0x00, 0x00, 0x00, 0x00,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x77, 0x00, 0x00, 0xFF,
	0x90, 0x00, 0x00, 0xFF, 0xA8, 0x00, 0x00, 0xFF, 0xAA, 0x00, 0x00, 0xFF, 0xB4, 0x00, 0x00, 0xFF,
	0xB0, 0x00, 0x00, 0xFF, 0x95, 0x00, 0x00, 0xFF, 0x6A, 0x00, 0x00, 0xFF, 0x59, 0x00, 0x00, 0xFF,
	0x3C, 0x00, 0x00, 0xFF, 0x13, 0x00, 0x00, 0xFF, 0x1A, 0x00, 0x00, 0xFF, 0x21, 0x00, 0x00, 0xFF,
	0x23, 0x00, 0x00, 0xFF, 0x1D, 0x00, 0x00, 0xFF, 0x1D, 0x00, 0x00, 0xFF, 0x39, 0x00, 0x00, 0xFF,
	0x72, 0x00, 0x00, 0xFF, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x89, 0x00, 0x00, 0xFF, 0x88, 0x00, 0x00, 0xFF,
	0xAA, 0x00, 0x00, 0xFF, 0xB5, 0x00, 0x00, 0xFF, 0xB8, 0x00, 0x00, 0xFF, 0xC9, 0x00, 0x00, 0xFF,
	0xC2, 0x00, 0x00, 0xFF, 0xA8, 0x00, 0x00, 0xFF, 0x7C, 0x00, 0x00, 0xFF, 0x71, 0x00, 0x00, 0xFF,
	0x3A, 0x00, 0x00, 0xFF, 0x1E, 0x00, 0x00, 0xFF, 0x1F, 0x00, 0x00, 0xFF, 0x1E, 0x00, 0x00, 0xFF,
	0x1F, 0x00, 0x00, 0xFF, 0x39, 0x00, 0x00, 0xFF, 0x51, 0x00, 0x00, 0xFF, 0x70, 0x00, 0x00, 0xFF,
	0x6F, 0x00, 0x00, 0xFF, 0x90, 0x00, 0x00, 0xFF, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x00, 0x00, 0x00, 0x00, 0x8D, 0x00, 0x00, 0xFF, 0xA3, 0x00, 0x00, 0xFF, 0xB1, 0x00, 0x00, 0xFF,
	0xC0, 0x00, 0x00, 0xFF, 0xC8, 0x00, 0x00, 0xFF, 0xCE, 0x00, 0x00, 0xFF, 0xDE, 0x00, 0x00, 0xFF,
	0xE1, 0x00, 0x00, 0xFF, 0xD4, 0x00, 0x00, 0xFF, 0xA6, 0x00, 0x00, 0xFF, 0x87, 0x00, 0x00, 0xFF,
	0x45, 0x00, 0x00, 0xFF, 0x2D, 0x00, 0x00, 0xFF, 0x1F, 0x00, 0x00, 0xFF, 0x2B, 0x00, 0x00, 0xFF,
	0x59, 0x00, 0x00, 0xFF, 0x6F, 0x00, 0x00, 0xFF, 0x5D, 0x00, 0x00, 0xFF, 0x38, 0x00, 0x00, 0xFF,
	0x62, 0x00, 0x00, 0xFF, 0x8E, 0x00, 0x00, 0xFF, 0xA3, 0x00, 0x00, 0xFF, 0x00, 0x00, 0x00, 0x00,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x8A, 0x00, 0x00, 0xFF, 0x8C, 0x00, 0x00, 0xFF, 0x9D, 0x00, 0x00, 0xFF, 0xAD, 0x00, 0x00, 0xFF,
	0xBA, 0x00, 0x00, 0xFF, 0xCA, 0x00, 0x00, 0xFF, 0xD3, 0x00, 0x00, 0xFF, 0xE4, 0x00, 0x00, 0xFF,
	0xF2, 0x00, 0x00, 0xFF, 0xF6, 0x00, 0x00, 0xFF, 0xD0, 0x00, 0x00, 0xFF, 0x99, 0x00, 0x00, 0xFF,
	0x63, 0x00, 0x00, 0xFF, 0x5A, 0x00, 0x00, 0xFF, 0x39, 0x00, 0x00, 0xFF, 0x32, 0x00, 0x00, 0xFF,
	0x49, 0x00, 0x00, 0xFF, 0x48, 0x00, 0x00, 0xFF, 0x26, 0x00, 0x00, 0xFF, 0x2E, 0x00, 0x00, 0xFF,
	0x4F, 0x00, 0x00, 0xFF, 0x61, 0x00, 0x00, 0xFF, 0x71, 0x00, 0x00, 0xFF, 0x79, 0x00, 0x00, 0xFF,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xA8, 0x00, 0x00, 0xFF,
	0xAC, 0x00, 0x00, 0xFF, 0xB8, 0x00, 0x00, 0xFF, 0xC0, 0x00, 0x00, 0xFF, 0xC7, 0x00, 0x00, 0xFF,
	0xDB, 0x00, 0x00, 0xFF, 0xE0, 0x00, 0x00, 0xFF, 0xE8, 0x00, 0x00, 0xFF, 0xF8, 0x00, 0x00, 0xFF,
	0xFF, 0x06, 0x06, 0xFF, 0xFF, 0x12, 0x12, 0xFF, 0xF5, 0x00, 0x00, 0xFF, 0xD1, 0x00, 0x00, 0xFF,
	0xBA, 0x00, 0x00, 0xFF, 0x8B, 0x00, 0x00, 0xFF, 0x3C, 0x00, 0x00, 0xFF, 0x1D, 0x00, 0x00, 0xFF,
	0x22, 0x00, 0x00, 0xFF, 0x31, 0x00, 0x00, 0xFF, 0x1D, 0x00, 0x00, 0xFF, 0x32, 0x00, 0x00, 0xFF,
	0x4F, 0x00, 0x00, 0xFF, 0x62, 0x00, 0x00, 0xFF, 0x79, 0x00, 0x00, 0xFF, 0x7B, 0x00, 0x00, 0xFF,
	0x85, 0x00, 0x00, 0xFF, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xA8, 0x00, 0x00, 0xFF, 0xB9, 0x00, 0x00, 0xFF,
	0xB0, 0x00, 0x00, 0xFF, 0xBE, 0x00, 0x00, 0xFF, 0xC7, 0x00, 0x00, 0xFF, 0xDE, 0x00, 0x00, 0xFF,
	0xFF, 0x01, 0x01, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0x19, 0x00, 0x00, 0xFF, 0x27, 0x00, 0x00, 0xFF,
	0x3A, 0x00, 0x00, 0xFF, 0x4E, 0x00, 0x00, 0xFF, 0x68, 0x00, 0x00, 0xFF, 0x64, 0x00, 0x00, 0xFF,
	0x67, 0x00, 0x00, 0xFF, 0x6A, 0x00, 0x00, 0xFF, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xC6, 0x00, 0x00, 0xFF, 0xCE, 0x00, 0x00, 0xFF,
	0xD8, 0x00, 0x00, 0xFF, 0xDB, 0x00, 0x00, 0xFF, 0xE0, 0x00, 0x00, 0xFF, 0xE4, 0x00, 0x00, 0xFF,
	0xE2, 0x00, 0x00, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
	0x1D, 0x00, 0x00, 0xFF, 0x30, 0x00, 0x00, 0xFF, 0x3C, 0x00, 0x00, 0xFF, 0x2F, 0x00, 0x00, 0xFF,
	0x33, 0x00, 0x00, 0xFF, 0x31, 0x00, 0x00, 0xFF, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x00, 0x00, 0x00, 0x00, 0xA8, 0x00, 0x00, 0xFF, 0xC0, 0x00, 0x00, 0xFF, 0xB6, 0x00, 0x00, 0xFF,
	0xB0, 0x00, 0x00, 0xFF, 0xB1, 0x00, 0x00, 0xFF, 0xB6, 0x00, 0x00, 0xFF, 0xBA, 0x00, 0x00, 0xFF,
	0xB9, 0x00, 0x00, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
	0xFF, 0xFF, 0xFF, 0xFF, 0x2A, 0x00, 0x00, 0xFF, 0x2D, 0x00, 0x00, 0xFF, 0x26, 0x00, 0x00, 0xFF,
	0x25, 0x00, 0x00, 0xFF, 0x35, 0x00, 0x00, 0xFF, 0x55, 0x00, 0x00, 0xFF, 0x00, 0x00, 0x00, 0x00,
	0x00, 0x00, 0x00, 0x00, 0x88, 0x00, 0x00, 0xFF, 0x76, 0x00, 0x00, 0xFF, 0x5F, 0x00, 0x00, 0xFF,
	0x61, 0x00, 0x00, 0xFF, 0x6C, 0x00, 0x00, 0xFF, 0x8D, 0x00, 0x00, 0xFF, 0x94, 0x00, 0x00, 0xFF,
	0x69, 0x00, 0x00, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
	0xFF, 0xFF, 0xFF, 0xFF, 0x47, 0x00, 0x00, 0xFF, 0x37, 0x00, 0x00, 0xFF, 0x17, 0x00, 0x00, 0xFF,
	0x17, 0x00, 0x00, 0xFF, 0x30, 0x00, 0x00, 0xFF, 0x44, 0x00, 0x00, 0xFF, 0x00, 0x00, 0x00, 0x00,
	0x00, 0x00, 0x00, 0x00, 0x59, 0x00, 0x00, 0xFF, 0x55, 0x00, 0x00, 0xFF, 0x42, 0x00, 0x00, 0xFF,
	0x36, 0x00, 0x00, 0xFF, 0x4C, 0x00, 0x00, 0xFF, 0x7A, 0x00, 0x00, 0xFF, 0x7E, 0x00, 0x00, 0xFF,
	0x3B, 0x00, 0x00, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0x2B, 0x00, 0x00, 0xFF, 0x4D, 0x00, 0x00, 0xFF,
	0x4C, 0x00, 0x00, 0xFF, 0x27, 0x00, 0x00, 0xFF, 0x4C, 0x00, 0x00, 0xFF, 0x7D, 0x00, 0x00, 0xFF,
	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
	0xFF, 0xFF, 0xFF, 0xFF, 0x6E, 0x00, 0x00, 0xFF, 0x3D, 0x00, 0x00, 0xFF, 0x18, 0x00, 0x00, 0xFF,
	0x10, 0x00, 0x00, 0xFF, 0x1B, 0x00, 0x00, 0xFF, 0x3F, 0x00, 0x00, 0xFF, 0x00, 0x00, 0x00, 0x00,
	0x36, 0x00, 0x00, 0xFF, 0x3A, 0x00, 0x00, 0xFF, 0x47, 0x00, 0x00, 0xFF, 0x57, 0x00, 0x00, 0xFF,
	0x5B, 0x00, 0x00, 0xFF, 0x62, 0x00, 0x00, 0xFF, 0x6B, 0x00, 0x00, 0xFF, 0x5B, 0x00, 0x00, 0xFF,
	0x17, 0x00, 0x00, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0x72, 0x00, 0x00, 0xFF, 0x71, 0x00, 0x00, 0xFF,
	0x4D, 0x00, 0x00, 0xFF, 0x80, 0x00, 0x00, 0xFF, 0xA9, 0x00, 0x00, 0xFF, 0xAC, 0x00, 0x00, 0xFF,
	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
	0xFF, 0xFF, 0xFF, 0xFF, 0x85, 0x00, 0x00, 0xFF, 0x43, 0x00, 0x00, 0xFF, 0x31, 0x00, 0x00, 0xFF,
	0x19, 0x00, 0x00, 0xFF, 0x11, 0x00, 0x00, 0xFF, 0x16, 0x00, 0x00, 0xFF, 0x31, 0x00, 0x00, 0xFF,
	0x35, 0x00, 0x00, 0xFF, 0x37, 0x00, 0x00, 0xFF, 0x3F, 0x00, 0x00, 0xFF, 0x4A, 0x00, 0x00, 0xFF,
	0x4C, 0x00, 0x00, 0xFF, 0x4D, 0x00, 0x00, 0xFF, 0x47, 0x00, 0x00, 0xFF, 0x2F, 0x00, 0x00, 0xFF,
	0x25, 0x00, 0x00, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
	0xFF, 0xFF, 0xFF, 0xFF, 0x84, 0x00, 0x00, 0xFF, 0x43, 0x00, 0x00, 0xFF, 0x34, 0x00, 0x00, 0xFF,
	0x35, 0x00, 0x00, 0xFF, 0x2A, 0x00, 0x00, 0xFF, 0x10, 0x00, 0x00, 0xFF, 0x1C, 0x00, 0x00, 0xFF,
	0x25, 0x00, 0x00, 0xFF, 0x23, 0x00, 0x00, 0xFF, 0x25, 0x00, 0x00, 0xFF, 0x27, 0x00, 0x00, 0xFF,
	0x26, 0x00, 0x00, 0xFF, 0x26, 0x00, 0x00, 0xFF, 0x23, 0x00, 0x00, 0xFF, 0x19, 0x00, 0x00, 0xFF,
	0x23, 0x00, 0x00, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
	0x84, 0x00, 0x00, 0xFF, 0x77, 0x00, 0x00, 0xFF, 0x54, 0x00, 0x00, 0xFF, 0x3B, 0x00, 0x00, 0xFF,
	0x21, 0x00, 0x00, 0xFF, 0x17, 0x00, 0x00, 0xFF, 0x13, 0x00, 0x00, 0xFF, 0x16, 0x00, 0x00, 0xFF,
	0x1B, 0x00, 0x00, 0xFF, 0x19, 0x00, 0x00, 0xFF, 0x24, 0x00, 0x00, 0xFF, 0x2F, 0x00, 0x00, 0xFF,
	0x38, 0x00, 0x00, 0xFF, 0x37, 0x00, 0x00, 0xFF, 0x2F, 0x00, 0x00, 0xFF, 0x21, 0x00, 0x00, 0xFF,
	0x1F, 0x00, 0x00, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xAC, 0x00, 0x00, 0xFF,
	0xAD, 0x00, 0x00, 0xFF, 0x9C, 0x00, 0x00, 0xFF, 0x7B, 0x00, 0x00, 0xFF, 0x53, 0x00, 0x00, 0xFF,
	0x39, 0x00, 0x00, 0xFF, 0x31, 0x00, 0x00, 0xFF, 0x20, 0x00, 0x00, 0xFF, 0x1D, 0x00, 0x00, 0xFF,
	0x2C, 0x00, 0x00, 0xFF, 0x2F, 0x00, 0x00, 0xFF, 0x35, 0x00, 0x00, 0xFF, 0x3B, 0x00, 0x00, 0xFF,
	0x40, 0x00, 0x00, 0xFF, 0x3D, 0x00, 0x00, 0xFF, 0x2D, 0x00, 0x00, 0xFF, 0x1C, 0x00, 0x00, 0xFF,
	0x27, 0x00, 0x00, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
	0xFF, 0xFF, 0xFF, 0xFF, 0xED, 0x00, 0x00, 0xFF, 0xD0, 0x00, 0x00, 0xFF, 0xCA, 0x00, 0x00, 0xFF,
	0xAE, 0x00, 0x00, 0xFF, 0x92, 0x00, 0x00, 0xFF, 0x79, 0x00, 0x00, 0xFF, 0x63, 0x00, 0x00, 0xFF,
	0x4D, 0x00, 0x00, 0xFF, 0x47, 0x00, 0x00, 0xFF, 0x33, 0x00, 0x00, 0xFF, 0x23, 0x00, 0x00, 0xFF,
	0x48, 0x00, 0x00, 0xFF, 0x4F, 0x00, 0x00, 0xFF, 0x41, 0x00, 0x00, 0xFF, 0x2F, 0x00, 0x00, 0xFF,
	0x2B, 0x00, 0x00, 0xFF, 0x1E, 0x00, 0x00, 0xFF, 0x17, 0x00, 0x00, 0xFF, 0x16, 0x00, 0x00, 0xFF,
	0x1D, 0x00, 0x00, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0x9E, 0x00, 0x00, 0xFF, 0xD6, 0x00, 0x00, 0xFF,
	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0x96, 0x00, 0x00, 0xFF, 0x98, 0x00, 0x00, 0xFF,
	0x85, 0x00, 0x00, 0xFF, 0x73, 0x00, 0x00, 0xFF, 0x56, 0x00, 0x00, 0xFF, 0x4B, 0x00, 0x00, 0xFF,
	0x37, 0x00, 0x00, 0xFF, 0x29, 0x00, 0x00, 0xFF, 0x21, 0x00, 0x00, 0xFF, 0x23, 0x00, 0x00, 0xFF,
	0x6A, 0x00, 0x00, 0xFF, 0x67, 0x00, 0x00, 0xFF, 0x3F, 0x00, 0x00, 0xFF, 0x1C, 0x00, 0x00, 0xFF,
	0x10, 0x00, 0x00, 0xFF, 0x16, 0x00, 0x00, 0xFF, 0x1F, 0x00, 0x00, 0xFF, 0x22, 0x00, 0x00, 0xFF,
	0x1A, 0x00, 0x00, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xC6, 0x00, 0x00, 0xFF, 0xDF, 0x00, 0x00, 0xFF,
	0xFF, 0x04, 0x04, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0x35, 0x00, 0x00, 0xFF,
	0x2B, 0x00, 0x00, 0xFF, 0x27, 0x00, 0x00, 0xFF, 0x27, 0x00, 0x00, 0xFF, 0x26, 0x00, 0x00, 0xFF,
	0x2D, 0x00, 0x00, 0xFF, 0x3F, 0x00, 0x00, 0xFF, 0x59, 0x00, 0x00, 0xFF, 0x65, 0x00, 0x00, 0xFF,
	0x5F, 0x00, 0x00, 0xFF, 0x56, 0x00, 0x00, 0xFF, 0x31, 0x00, 0x00, 0xFF, 0x16, 0x00, 0x00, 0xFF,
	0x11, 0x00, 0x00, 0xFF, 0x16, 0x00, 0x00, 0xFF, 0x22, 0x00, 0x00, 0xFF, 0x38, 0x00, 0x00, 0xFF,
	0x40, 0x00, 0x00, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xC1, 0x00, 0x00, 0xFF, 0xC0, 0x00, 0x00, 0xFF,
	0xCD, 0x00, 0x00, 0xFF, 0xCB, 0x00, 0x00, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0x16, 0x00, 0x00, 0xFF,
	0x25, 0x00, 0x00, 0xFF, 0x38, 0x00, 0x00, 0xFF, 0x4D, 0x00, 0x00, 0xFF, 0x54, 0x00, 0x00, 0xFF,
	0x5D, 0x00, 0x00, 0xFF, 0x79, 0x00, 0x00, 0xFF, 0x8D, 0x00, 0x00, 0xFF, 0x8A, 0x00, 0x00, 0xFF,
	0x00, 0x00, 0x00, 0x00, 0x13, 0x00, 0x00, 0xFF, 0x16, 0x00, 0x00, 0xFF, 0x0F, 0x00, 0x00, 0xFF,
	0x13, 0x00, 0x00, 0xFF, 0x2B, 0x00, 0x00, 0xFF, 0x4A, 0x00, 0x00, 0xFF, 0x5B, 0x00, 0x00, 0xFF,
	0x77, 0x00, 0x00, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0x91, 0x00, 0x00, 0xFF, 0x8E, 0x00, 0x00, 0xFF,
	0x75, 0x00, 0x00, 0xFF, 0x66, 0x00, 0x00, 0xFF, 0x50, 0x00, 0x00, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
	0x28, 0x00, 0x00, 0xFF, 0x3B, 0x00, 0x00, 0xFF, 0x5B, 0x00, 0x00, 0xFF, 0x76, 0x00, 0x00, 0xFF,
	0xAC, 0x00, 0x00, 0xFF, 0xBB, 0x00, 0x00, 0xFF, 0xD2, 0x00, 0x00, 0xFF, 0x00, 0x00, 0x00, 0x00,
	0x00, 0x00, 0x00, 0x00, 0x14, 0x00, 0x00, 0xFF, 0x0F, 0x00, 0x00, 0xFF, 0x18, 0x00, 0x00, 0xFF,
	0x24, 0x00, 0x00, 0xFF, 0x42, 0x00, 0x00, 0xFF, 0x47, 0x00, 0x00, 0xFF, 0x6E, 0x00, 0x00, 0xFF,
	0x91, 0x00, 0x00, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0x76, 0x00, 0x00, 0xFF, 0x77, 0x00, 0x00, 0xFF,
	0x64, 0x00, 0x00, 0xFF, 0x62, 0x00, 0x00, 0xFF, 0x3B, 0x00, 0x00, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
	0x31, 0x00, 0x00, 0xFF, 0x4F, 0x00, 0x00, 0xFF, 0x7B, 0x00, 0x00, 0xFF, 0xA8, 0x00, 0x00, 0xFF,
	0xD1, 0x00, 0x00, 0xFF, 0xDD, 0x00, 0x00, 0xFF, 0xE2, 0x00, 0x00, 0xFF, 0x00, 0x00, 0x00, 0x00,
	0x00, 0x00, 0x00, 0x00, 0x36, 0x00, 0x00, 0xFF, 0x25, 0x00, 0x00, 0xFF, 0x15, 0x00, 0x00, 0xFF,
	0x42, 0x00, 0x00, 0xFF, 0x5B, 0x00, 0x00, 0xFF, 0x6E, 0x00, 0x00, 0xFF, 0x81, 0x00, 0x00, 0xFF,
	0x91, 0x00, 0x00, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0x61, 0x00, 0x00, 0xFF, 0x59, 0x00, 0x00, 0xFF,
	0x59, 0x00, 0x00, 0xFF, 0x37, 0x00, 0x00, 0xFF, 0x15, 0x00, 0x00, 0xFF, 0x14, 0x00, 0x00, 0xFF,
	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
	0xFF, 0xFF, 0xFF, 0xFF, 0x7A, 0x00, 0x00, 0xFF, 0xA9, 0x00, 0x00, 0xFF, 0xC5, 0x00, 0x00, 0xFF,
	0xE7, 0x00, 0x00, 0xFF, 0xF3, 0x00, 0x00, 0xFF, 0xE0, 0x00, 0x00, 0xFF, 0x00, 0x00, 0x00, 0x00,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x2D, 0x00, 0x00, 0xFF, 0x19, 0x00, 0x00, 0xFF,
	0x4B, 0x00, 0x00, 0xFF, 0x6F, 0x00, 0x00, 0xFF, 0x88, 0x00, 0x00, 0xFF, 0x90, 0x00, 0x00, 0xFF,
	0xA3, 0x00, 0x00, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0x4B, 0x00, 0x00, 0xFF, 0x54, 0x00, 0x00, 0xFF,
	0x2F, 0x00, 0x00, 0xFF, 0x13, 0x00, 0x00, 0xFF, 0x1B, 0x00, 0x00, 0xFF, 0x24, 0x00, 0x00, 0xFF,
	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xCE, 0x00, 0x00, 0xFF, 0xEC, 0x00, 0x00, 0xFF,
	0xFF, 0x09, 0x09, 0xFF, 0xFF, 0x09, 0x09, 0xFF, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x35, 0x00, 0x00, 0xFF, 0x1F, 0x00, 0x00, 0xFF,
	0x67, 0x00, 0x00, 0xFF, 0x84, 0x00, 0x00, 0xFF, 0x8C, 0x00, 0x00, 0xFF, 0x8D, 0x00, 0x00, 0xFF,
	0x71, 0x00, 0x00, 0xFF, 0x3D, 0x00, 0x00, 0xFF, 0x35, 0x00, 0x00, 0xFF, 0x2B, 0x00, 0x00, 0xFF,
	0x34, 0x00, 0x00, 0xFF, 0x35, 0x00, 0x00, 0xFF, 0x27, 0x00, 0x00, 0xFF, 0x2F, 0x00, 0x00, 0xFF,
	0x19, 0x00, 0x00, 0xFF, 0x21, 0x00, 0x00, 0xFF, 0x3C, 0x00, 0x00, 0xFF, 0x51, 0x00, 0x00, 0xFF,
	0x69, 0x00, 0x00, 0xFF, 0x75, 0x00, 0x00, 0xFF, 0x94, 0x00, 0x00, 0xFF, 0xB5, 0x00, 0x00, 0xFF,
	0xC6, 0x00, 0x00, 0xFF, 0xCC, 0x00, 0x00, 0xFF, 0xE4, 0x00, 0x00, 0xFF, 0xF4, 0x00, 0x00, 0xFF,
	0xFF, 0x05, 0x05, 0xFF, 0xEA, 0x00, 0x00, 0xFF, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x3F, 0x00, 0x00, 0xFF,
	0x75, 0x00, 0x00, 0xFF, 0x8D, 0x00, 0x00, 0xFF, 0x6F, 0x00, 0x00, 0xFF, 0x5D, 0x00, 0x00, 0xFF,
	0x36, 0x00, 0x00, 0xFF, 0x3C, 0x00, 0x00, 0xFF, 0x43, 0x00, 0x00, 0xFF, 0x26, 0x00, 0x00, 0xFF,
	0x38, 0x00, 0x00, 0xFF, 0x37, 0x00, 0x00, 0xFF, 0x1B, 0x00, 0x00, 0xFF, 0x15, 0x00, 0x00, 0xFF,
	0x15, 0x00, 0x00, 0xFF, 0x2F, 0x00, 0x00, 0xFF, 0x5B, 0x00, 0x00, 0xFF, 0x73, 0x00, 0x00, 0xFF,
	0x8C, 0x00, 0x00, 0xFF, 0x9E, 0x00, 0x00, 0xFF, 0xBC, 0x00, 0x00, 0xFF, 0xD0, 0x00, 0x00, 0xFF,
	0xDA, 0x00, 0x00, 0xFF, 0xEA, 0x00, 0x00, 0xFF, 0xFF, 0x02, 0x02, 0xFF, 0xFF, 0x05, 0x05, 0xFF,
	0xEC, 0x00, 0x00, 0xFF, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x79, 0x00, 0x00, 0xFF, 0x6B, 0x00, 0x00, 0xFF, 0x35, 0x00, 0x00, 0xFF, 0x20, 0x00, 0x00, 0xFF,
	0x46, 0x00, 0x00, 0xFF, 0x60, 0x00, 0x00, 0xFF, 0x3D, 0x00, 0x00, 0xFF, 0x23, 0x00, 0x00, 0xFF,
	0x19, 0x00, 0x00, 0xFF, 0x13, 0x00, 0x00, 0xFF, 0x15, 0x00, 0x00, 0xFF, 0x29, 0x00, 0x00, 0xFF,
	0x53, 0x00, 0x00, 0xFF, 0x65, 0x00, 0x00, 0xFF, 0x77, 0x00, 0x00, 0xFF, 0x86, 0x00, 0x00, 0xFF,
	0x9E, 0x00, 0x00, 0xFF, 0xB6, 0x00, 0x00, 0xFF, 0xD8, 0x00, 0x00, 0xFF, 0xDE, 0x00, 0x00, 0xFF,
	0xF0, 0x00, 0x00, 0xFF, 0xEF, 0x00, 0x00, 0xFF, 0xFF, 0x03, 0x03, 0xFF, 0xF2, 0x00, 0x00, 0xFF,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x00, 0x00, 0x00, 0x00, 0x4C, 0x00, 0x00, 0xFF, 0x15, 0x00, 0x00, 0xFF, 0x1A, 0x00, 0x00, 0xFF,
	0x65, 0x00, 0x00, 0xFF, 0x6C, 0x00, 0x00, 0xFF, 0x3C, 0x00, 0x00, 0xFF, 0x19, 0x00, 0x00, 0xFF,
	0x0E, 0x00, 0x00, 0xFF, 0x09, 0x00, 0x00, 0xFF, 0x26, 0x00, 0x00, 0xFF, 0x3D, 0x00, 0x00, 0xFF,
	0x49, 0x00, 0x00, 0xFF, 0x4F, 0x00, 0x00, 0xFF, 0x60, 0x00, 0x00, 0xFF, 0x7E, 0x00, 0x00, 0xFF,
	0x9C, 0x00, 0x00, 0xFF, 0xBE, 0x00, 0x00, 0xFF, 0xDD, 0x00, 0x00, 0xFF, 0xEB, 0x00, 0x00, 0xFF,
	0xFB, 0x00, 0x00, 0xFF, 0xF2, 0x00, 0x00, 0xFF, 0xF0, 0x00, 0x00, 0xFF, 0x00, 0x00, 0x00, 0x00,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x2A, 0x00, 0x00, 0xFF, 0x4D, 0x00, 0x00, 0xFF,
	0x80, 0x00, 0x00, 0xFF, 0x60, 0x00, 0x00, 0xFF, 0x28, 0x00, 0x00, 0xFF, 0x1A, 0x00, 0x00, 0xFF,
	0x0D, 0x00, 0x00, 0xFF, 0x10, 0x00, 0x00, 0xFF, 0x12, 0x00, 0x00, 0xFF, 0x19, 0x00, 0x00, 0xFF,
	0x25, 0x00, 0x00, 0xFF, 0x49, 0x00, 0x00, 0xFF, 0x71, 0x00, 0x00, 0xFF, 0x94, 0x00, 0x00, 0xFF,
	0xAC, 0x00, 0x00, 0xFF, 0xC2, 0x00, 0x00, 0xFF, 0xD3, 0x00, 0x00, 0xFF, 0xE3, 0x00, 0x00, 0xFF,
	0xE6, 0x00, 0x00, 0xFF, 0xDA, 0x00, 0x00, 0xFF, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x00, 0x00, 0xFF,
	0x88, 0x00, 0x00, 0xFF, 0x50, 0x00, 0x00, 0xFF, 0x25, 0x00, 0x00, 0xFF, 0x14, 0x00, 0x00, 0xFF,
	0x0B, 0x00, 0x00, 0xFF, 0x10, 0x00, 0x00, 0xFF, 0x12, 0x00, 0x00, 0xFF, 0x14, 0x00, 0x00, 0xFF,
	0x29, 0x00, 0x00, 0xFF, 0x62, 0x00, 0x00, 0xFF, 0x7B, 0x00, 0x00, 0xFF, 0x8F, 0x00, 0x00, 0xFF,
	0xAB, 0x00, 0x00, 0xFF, 0xC1, 0x00, 0x00, 0xFF, 0xD9, 0x00, 0x00, 0xFF, 0xDB, 0x00, 0x00, 0xFF,
	0xBC, 0x00, 0x00, 0xFF, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x00, 0x00, 0x00, 0x00, 0x4F, 0x00, 0x00, 0xFF, 0x29, 0x00, 0x00, 0xFF, 0x16, 0x00, 0x00, 0xFF,
	0x0F, 0x00, 0x00, 0xFF, 0x15, 0x00, 0x00, 0xFF, 0x29, 0x00, 0x00, 0xFF, 0x2D, 0x00, 0x00, 0xFF,
	0x45, 0x00, 0x00, 0xFF, 0x68, 0x00, 0x00, 0xFF, 0x80, 0x00, 0x00, 0xFF, 0x95, 0x00, 0x00, 0xFF,
	0xA9, 0x00, 0x00, 0xFF, 0xBA, 0x00, 0x00, 0xFF, 0xBB, 0x00, 0x00, 0xFF, 0x00, 0x00, 0x00, 0x00,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x2A, 0x00, 0x00, 0xFF, 0x2D, 0x00, 0x00, 0xFF, 0x39, 0x00, 0x00, 0xFF, 0x39, 0x00, 0x00, 0xFF,
	0x50, 0x00, 0x00, 0xFF, 0x61, 0x00, 0x00, 0xFF, 0x69, 0x00, 0x00, 0xFF, 0x86, 0x00, 0x00, 0xFF,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00
};

SDL_Surface* IconSurface = NULL;
#endif

void VID_PrepareModeList(void);

//Hudler: 16/10/99: added for OpenGL gamma correction
RGBA_t  gamma_correction = {0x7F7F7F7F};
extern consvar_t cv_grgammared;
extern consvar_t cv_grgammagreen;
extern consvar_t cv_grgammablue;

extern consvar_t cv_fullscreen; // for fullscreen support

static int numVidModes= 0;

static char vidModeName[33][32]; // allow 33 different modes

boolean highcolor = false;
boolean mousegrabbed = false;

// synchronize page flipping with screen refresh
// unused and for compatibilityy reason
consvar_t       cv_vidwait = {"vid_wait","1",CV_SAVE,CV_OnOff};

byte graphics_started = 0; // Is used in console.c and screen.c

// To disable fullscreen at startup; is set in VID_PrepareModeList
boolean allow_fullscreen = false;

event_t myEvent;

// SDL vars
static       SDL_Surface *vidSurface=NULL;
static       SDL_Color    localPalette[256];
static       SDL_Rect   **modeList=NULL;
static       Uint8        BitsPerPixel;
const static Uint32       surfaceFlags = SDL_HWSURFACE|SDL_HWPALETTE|SDL_DOUBLEBUF;

// first entry in the modelist which is not bigger than 1024x768
static int firstEntry=0;

// windowed video modes from which to choose from.

static int windowedModes[][2] =
{
	{2560, 2048},	// QSXGA
	{2560, 1600},	// WQXGA
	{2048, 1536},	// QXGA
	{2048, 1080},	// 2K
	{1920, 1080},	// HD 1080
	{1680, 1050},	// WXSGA+
	{1440, 960},	// --
	{1440, 900},	// --
	{1400, 1050},	// SXGA+
	{1280, 1024},	// SXGA
	{1280, 1000},	// --
	{1280, 960},	// --
	{1280, 854},	// --
	{1280, 800},	// WXGA
	{1280, 768},	// WXGA
	{1280, 720},	// HD 720
	{1152, 768},	// --
	{1024, 800},	// --
	{1024, 768},	// XGA
	{1024, 600},	// WSVGA
	{960, 720},		// --
	{800, 600},		// SVGA
	{800, 480},		// WVGA
	{768, 576},		// PAL
	{720, 480},		// NTSC
	{640, 480},		// VGA
	{640, 400},		// --
	{512, 400},		// --
	{512, 384},		// --
	{400, 300},		// --
	{320, 240},		// QVGA
	{320, 320},		// --
	{320, 200},		// CGA
	{256, 200},		// --
	{160, 160},		// --
};

// maximum number of windowed modes (see windowedModes[][])
#define MAXWINMODES (sizeof(windowedModes) / sizeof(*windowedModes))


//
//  Translates the SDL key into Doom key
//

static int xlatekey(SDLKey sym)
{
    int rc = 0;

    switch(sym)
    {
    case SDLK_LEFT:  rc = KEY_LEFTARROW;     break;
    case SDLK_RIGHT: rc = KEY_RIGHTARROW;    break;
    case SDLK_DOWN:  rc = KEY_DOWNARROW;     break;
    case SDLK_UP:    rc = KEY_UPARROW;       break;

    case SDLK_ESCAPE:   rc = KEY_ESCAPE;        break;
    case SDLK_RETURN:   rc = KEY_ENTER;         break;
    case SDLK_TAB:      rc = KEY_TAB;           break;
    case SDLK_F1:       rc = KEY_F1;            break;
    case SDLK_F2:       rc = KEY_F2;            break;
    case SDLK_F3:       rc = KEY_F3;            break;
    case SDLK_F4:       rc = KEY_F4;            break;
    case SDLK_F5:       rc = KEY_F5;            break;
    case SDLK_F6:       rc = KEY_F6;            break;
    case SDLK_F7:       rc = KEY_F7;            break;
    case SDLK_F8:       rc = KEY_F8;            break;
    case SDLK_F9:       rc = KEY_F9;            break;
    case SDLK_F10:      rc = KEY_F10;           break;
    case SDLK_F11:      rc = KEY_F11;           break;
    case SDLK_F12:      rc = KEY_F12;           break;

    case SDLK_BACKSPACE: rc = KEY_BACKSPACE;    break;
    case SDLK_DELETE:    rc = KEY_DEL;          break;

    case SDLK_PAUSE:     rc = KEY_PAUSE;        break;

    case SDLK_EQUALS:
    case SDLK_PLUS:      rc = KEY_EQUALS;       break;

    case SDLK_MINUS:     rc = KEY_MINUS;        break;

    case SDLK_LSHIFT:
    case SDLK_RSHIFT:
        rc = KEY_SHIFT;
        break;

	case SDLK_CAPSLOCK:
		rc = KEY_CAPSLOCK;
		break;

    case SDLK_LCTRL:
    case SDLK_RCTRL:
        rc = KEY_CTRL;
        break;

    case SDLK_LALT:
    case SDLK_RALT:
        rc = KEY_ALT;
        break;

    case SDLK_PAGEUP:   rc = KEY_PGUP; break;
    case SDLK_PAGEDOWN: rc = KEY_PGDN; break;
    case SDLK_END:      rc = KEY_END;  break;
    case SDLK_HOME:     rc = KEY_HOME; break;
    case SDLK_INSERT:   rc = KEY_INS;  break;

    case SDLK_KP0: rc = KEY_KEYPAD0;  break;
    case SDLK_KP1: rc = KEY_KEYPAD1;  break;
    case SDLK_KP2: rc = KEY_KEYPAD2;  break;
    case SDLK_KP3: rc = KEY_KEYPAD3;  break;
    case SDLK_KP4: rc = KEY_KEYPAD4;  break;
    case SDLK_KP5: rc = KEY_KEYPAD5;  break;
    case SDLK_KP6: rc = KEY_KEYPAD6;  break;
    case SDLK_KP7: rc = KEY_KEYPAD7;  break;
    case SDLK_KP8: rc = KEY_KEYPAD8;  break;
    case SDLK_KP9: rc = KEY_KEYPAD9;  break;

    case SDLK_KP_MINUS:  rc = KEY_KPADDEL;  break;
    case SDLK_KP_DIVIDE: rc = KEY_KPADSLASH; break;
    case SDLK_KP_ENTER:  rc = KEY_ENTER;    break;

    default:
        if (sym >= SDLK_SPACE && sym <= SDLK_DELETE)
            rc = sym - SDLK_SPACE + ' ';
        if (sym >= 'A' && sym <= 'Z')
            rc = sym - 'A' + 'a';
        break;
    }

    return rc;
}



//
// I_StartFrame
//
void I_StartFrame(void)
{
    if(SDL_MUSTLOCK(vidSurface))
        if(SDL_LockSurface(vidSurface) < 0)
            return;
}

static int      lastmousex = 0;
static int      lastmousey = 0;

#ifdef LJOYSTICK
extern void I_GetJoyEvent();
#endif
#ifdef LMOUSE2
extern void I_GetMouse2Event();
#endif


void I_GetEvent(void)
{
    SDL_Event inputEvent;

#ifdef LJOYSTICK
    I_GetJoyEvent();
#endif
#ifdef LMOUSE2
    I_GetMouse2Event();
#endif

    //SDL_PumpEvents(); // FIXME: do we need this?

    while(SDL_PollEvent(&inputEvent))
    {
        switch(inputEvent.type)
        {
        case SDL_KEYDOWN:
            myEvent.type = ev_keydown;
            myEvent.data1 = xlatekey(inputEvent.key.keysym.sym);
            D_PostEvent(&myEvent);
            break;
        case SDL_KEYUP:
            myEvent.type = ev_keyup;
            myEvent.data1 = xlatekey(inputEvent.key.keysym.sym);
            D_PostEvent(&myEvent);
            break;
        case SDL_MOUSEMOTION:
            if(cv_usemouse.value)
            {
                // If the event is from warping the pointer back to middle
                // of the screen then ignore it.
                if ((inputEvent.motion.x == vid.width/2) &&
                    (inputEvent.motion.y == vid.height/2))
                {
                    lastmousex = inputEvent.motion.x;
                    lastmousey = inputEvent.motion.y;
                    break;
                }
                else
                {
                    myEvent.data2 = (inputEvent.motion.x - lastmousex);// << 2;
                    lastmousex = inputEvent.motion.x;
                    myEvent.data3 = (lastmousey - inputEvent.motion.y);// << 2;
                    lastmousey = inputEvent.motion.y;
                }
                myEvent.type = ev_mouse;
                myEvent.data1 = 0;

                D_PostEvent(&myEvent);

                // Warp the pointer back to the middle of the window
                //  or we cannot move any further if it's at a border.
                /*if ((inputEvent.motion.x < (vid.width/2)-(vid.width/4)) ||
                    (inputEvent.motion.y < (vid.height/2)-(vid.height/4)) ||
                    (inputEvent.motion.x > (vid.width/2)+(vid.width/4)) ||
                    (inputEvent.motion.y > (vid.height/2)+(vid.height/4)))
                {*/
                if (!menuactive && !consoleready && !demoplayback)
                {
                    SDL_WarpMouse(vid.width/2, vid.height/2);
                    SDL_ShowCursor(SDL_DISABLE);
                }
                else
                	SDL_ShowCursor(SDL_ENABLE);
                //}
            }
            break;
        case SDL_MOUSEBUTTONDOWN:
            if(cv_usemouse.value)
            {
                myEvent.type = ev_keydown;
		if(inputEvent.button.button==4)
		{
			myEvent.data1 = KEY_MOUSE1WHEELUP;
		}
		else if(inputEvent.button.button==5)
		{
			myEvent.data1 = KEY_MOUSE1WHEELDOWN;
		}
		else
		{
                	myEvent.data1 = KEY_MOUSE1B1 + (inputEvent.button.button - 1); // FIXME!
		}
                D_PostEvent(&myEvent);
            }
            break;
        case SDL_MOUSEBUTTONUP:
            if(cv_usemouse.value)
            {
                myEvent.type = ev_keyup;
		if(inputEvent.button.button==4||inputEvent.button.button==5)
		{
			//ignore wheel
		}
		else
		{
                	myEvent.data1 = KEY_MOUSE1B1 + (inputEvent.button.button - 1); // FIXME!
			D_PostEvent(&myEvent);
		}
            }
            break;

        case SDL_QUIT:
            M_QuitResponse('y');
            break;
        default:
            break;

        }
    }
}

static void doGrabMouse(void)
{
	if (cv_usemouse.value)
		if(SDL_GRAB_OFF == SDL_WM_GrabInput(SDL_GRAB_QUERY))
		{
			SDL_WM_GrabInput(SDL_GRAB_ON);
			mousegrabbed = true;
		}
}

static void doUngrabMouse(void)
{
	if (cv_usemouse.value)
		if(SDL_GRAB_ON == SDL_WM_GrabInput(SDL_GRAB_QUERY))
		{
			SDL_WM_GrabInput(SDL_GRAB_OFF);
			mousegrabbed = false;
		}
}

void I_StartupMouse(void)
{
    SDL_Event inputEvent;

	if (cv_usemouse.value == 0 || M_CheckParm("-nomouse"))
	{
		cv_usemouse.value = 0;
		return;
	}

    // warp to center
    SDL_WarpMouse(vid.width/2, vid.height/2);
    lastmousex = vid.width/2;
    lastmousey = vid.height/2;
    // remove the mouse event by reading the queue
    SDL_PollEvent(&inputEvent);

#ifdef HAS_SDL_BEEN_FIXED // FIXME
  if(cv_usemouse.value)
    {
      doGrabMouse();
    }
  else
    {
      doUngrabMouse();
    }
#endif
    return;
}

/* I_UpdateJoysticks() -- Starts the joystick subsystem */
void I_UpdateJoysticks(void)
{
	// First Check
}

//
// I_OsPolling
//
void I_OsPolling(void)
{
    if (!graphics_started)
        return;

    I_GetEvent();

    //reset wheel like in win32, I don't understand it but works
    gamekeydown[KEY_MOUSE1WHEELUP] = 0;
    gamekeydown[KEY_MOUSE1WHEELDOWN] = 0;

    return;
}


//
// I_UpdateNoBlit
//
void I_UpdateNoBlit(void)
{
    /* this function intentionally left empty */
}

//
// I_FinishUpdate
//
void I_FinishUpdate(void)
{
    if(screens[0] != vid.direct)
    {
        memcpy(vid.direct, screens[0], vid.width*vid.height*vid.bpp);
        //screens[0] = vid.direct; //FIXME: we MUST render directly into the surface
    }

    //SDL_Flip(vidSurface);
    SDL_UpdateRect(vidSurface, 0, 0, 0, 0);

    if(SDL_MUSTLOCK(vidSurface))
    {
        SDL_UnlockSurface(vidSurface);
    }

    I_GetEvent();
}


//
// I_ReadScreen
//
void I_ReadScreen(byte* scr)
{
    memcpy (scr, screens[0], vid.width*vid.height*vid.bpp);
}



//
// I_SetPalette
//
void I_SetPalette(RGBA_t* palette)
{
    int i;

    for(i=0; i<256; i++)
    {
        localPalette[i].r = palette[i].s.red;
        localPalette[i].g = palette[i].s.green;
        localPalette[i].b = palette[i].s.blue;
    }

    SDL_SetColors(vidSurface, localPalette, 0, 256);

    return;
}


// return number of fullscreen + X11 modes
int   VID_NumModes(void)
{
    if(cv_fullscreen.value)
        return numVidModes - firstEntry;
    else
        return MAXWINMODES;
}

char  *VID_GetModeName(int modeNum)
{
    if(cv_fullscreen.value) { // fullscreen modes
        modeNum += firstEntry;
        if(modeNum >= numVidModes)
            return NULL;

        sprintf(&vidModeName[modeNum][0], "%dx%d",
                modeList[modeNum]->w,
                modeList[modeNum]->h);
    }
    else { // windowed modes
        if(modeNum > MAXWINMODES)
            return NULL;

        sprintf(&vidModeName[modeNum][0], "%dx%d",
                windowedModes[modeNum][0],
                windowedModes[modeNum][1]);
    }
    return &vidModeName[modeNum][0];
}

int VID_GetModeForSize(int w, int h) {
    int matchMode, i;

    if(cv_fullscreen.value)
    {
        matchMode=-1;

        for(i=firstEntry; i<numVidModes; i++)
        {
            if(modeList[i]->w == w &&
               modeList[i]->h == h)
            {
                matchMode = i;
                break;
            }
        }
        if(-1 == matchMode) // use smallest mode
        {
            matchMode = numVidModes-1;
        }
        matchMode -= firstEntry;
    }
    else
    {
        matchMode=-1;

        for(i=0; i<MAXWINMODES; i++)
        {
            if(windowedModes[i][0] == w &&
               windowedModes[i][1] == h)
            {
                matchMode = i;
                break;
            }
        }

        if(-1 == matchMode) // use smallest mode
        {
            matchMode = MAXWINMODES-1;
        }
    }

    return matchMode;
}


void VID_PrepareModeList(void)
{
    int i;

	if(cv_fullscreen.value) // only fullscreen needs preparation
		if(numVidModes != -1)
			for(i = 0; i < numVidModes; i++)
			{
				firstEntry = i;
				break;
			}

	allow_fullscreen = true;
	return;
}

int VID_SetMode(int modeNum)
{
#ifdef _WIN32
	HICON hIconBig;
	HICON hIconSmall;
	HWND hWindow;
	SDL_SysWMinfo WinFo;
	int WMret = 0;
#endif

    doUngrabMouse();

    if(cv_fullscreen.value)
    {
        modeNum += firstEntry;

        vid.width = modeList[modeNum]->w;
        vid.height = modeList[modeNum]->h;
        vid.rowbytes = vid.width * vid.bpp;
        vid.recalc = true;
        SDL_FreeSurface(vidSurface);
        if (vid.buffer)
        {
        	free(vid.buffer);
        	vid.buffer = NULL;
        }

        vidSurface = SDL_SetVideoMode(vid.width, vid.height, BitsPerPixel, surfaceFlags | SDL_FULLSCREEN);
        if(NULL == vidSurface)
        {
            I_Error("Could not set vidmode\n");
        }

        vid.buffer = malloc(vid.width * vid.height * vid.bpp * NUMSCREENS);

        vid.direct = vidSurface->pixels; // FIXME
		vid.modenum = modeNum-firstEntry;
    }
    else //(cv_fullscreen.value)
    {
        vid.width = windowedModes[modeNum][0];
        vid.height = windowedModes[modeNum][1];
        vid.rowbytes = vid.width * vid.bpp;
        vid.recalc = true;

		// Window title
		SDL_WM_SetCaption("ReMooD "REMOOD_VERSIONSTRING" \""REMOOD_VERSIONCODESTRING"\"", "ReMooD");

#ifndef _WIN32
		if (!IconSurface)
		{
			UInt8 r[4] = {255, 0, 0, 0};
			UInt8 g[4] = {0, 255, 0, 0};
			UInt8 b[4] = {0, 0, 255, 0};
			UInt8 a[4] = {0, 0, 0, 255};
			IconSurface = SDL_CreateRGBSurfaceFrom(ICON, 32, 32, 32, 4 * 32, *((UInt32*)r), *((UInt32*)g), *((UInt32*)b), *((UInt32*)a));
		}
		
		SDL_WM_SetIcon(IconSurface, NULL);
#endif

        SDL_FreeSurface(vidSurface);
        if (vid.buffer)
        {
        	free(vid.buffer);
        	vid.buffer = NULL;
        }

        vidSurface = SDL_SetVideoMode(vid.width, vid.height, BitsPerPixel, surfaceFlags);

        if(NULL == vidSurface)
        {
            I_Error("Could not set vidmode\n");
        }

#ifdef _WIN32
		SDL_VERSION(&WinFo.version);
		if ((WMret = SDL_GetWMInfo(&WinFo)) == 1)
		{
			hIconBig = LoadIcon(GetModuleHandle(NULL), MAKEINTRESOURCE(1));
			hIconSmall = LoadIcon(GetModuleHandle(NULL), MAKEINTRESOURCE(2));
			hWindow = WinFo.window;
#if defined(_MSC_VER) && (_MSC_VER <= 1200)			// VC 6 does NOT have the following
			SetClassLong(hWindow, GCL_HICON, (LONG)hIconBig);
			SetClassLong(hWindow, GCL_HICONSM, (LONG)hIconBig);
#else
			SetClassLongPtr(hWindow, GCLP_HICON, (LONG_PTR) hIconBig);
			SetClassLongPtr(hWindow, GCLP_HICONSM, (LONG_PTR) hIconSmall);
			CONS_Printf("Set window icon.\n");
#endif
		}
		else
			CONS_Printf("Could not set window icon (%i).\n", WMret);
#endif

        vid.buffer = malloc(vid.width * vid.height * vid.bpp * NUMSCREENS);
        vid.direct = vidSurface->pixels; // FIXME
		vid.modenum = modeNum;
    }

    I_StartupMouse();

    return 1;
}

void I_StartupGraphics(void)
{
#ifdef _WIN32
	HICON hIconBig;
	HICON hIconSmall;
	HWND hWindow;
	SDL_SysWMinfo WinFo;
	int WMret = 0;
#endif

    if(graphics_started)
        return;

    CV_RegisterVar (&cv_vidwait);

    // Initialize Audio as well, otherwise DirectX can not use audio
    if(SDL_Init(SDL_INIT_AUDIO|SDL_INIT_VIDEO) < 0)
    {
        CONS_Printf("Couldn't initialize SDL: %s\n", SDL_GetError());
        return;
    }
    
    // Set color depth; either 1=256pseudocolor or 2=hicolor
    if (M_CheckParm("-highcolor"))
    {
    	vid.bpp = 2;
    	BitsPerPixel = 16;
    }
    else
    {
	    vid.bpp = 1 /*videoInfo->vfmt->BytesPerPixel*/;
	    BitsPerPixel = 8;
	}
    highcolor = (vid.bpp == 2) ? true:false;

    modeList = SDL_ListModes(NULL, SDL_FULLSCREEN|surfaceFlags);

    if(NULL == modeList)
    {
        CONS_Printf("No video modes present\n");
        return;
    }

    numVidModes=0;
    if(NULL != modeList)
    {
        while(NULL != modeList[numVidModes])
            numVidModes++;
    }
    else
        // should not happen with fullscreen modes
        numVidModes = -1;

    //CONS_Printf("Found %d Video Modes\n", numVidModes);

    // default size for startup
    vid.width = BASEVIDWIDTH;
    vid.height = BASEVIDHEIGHT;
    vid.rowbytes = vid.width * vid.bpp;
    vid.recalc = true;

    // Window title
    SDL_WM_SetCaption("ReMooD "REMOOD_VERSIONSTRING" \""REMOOD_VERSIONCODESTRING"\"", "ReMooD");
        
#ifndef _WIN32
		if (!IconSurface)
		{
			UInt8 r[4] = {255, 0, 0, 0};
			UInt8 g[4] = {0, 255, 0, 0};
			UInt8 b[4] = {0, 0, 255, 0};
			UInt8 a[4] = {0, 0, 0, 255};
			IconSurface = SDL_CreateRGBSurfaceFrom(ICON, 32, 32, 32, 4 * 32, *((UInt32*)r), *((UInt32*)g), *((UInt32*)b), *((UInt32*)a));
		}
		
		SDL_WM_SetIcon(IconSurface, NULL);
#endif

    vidSurface = SDL_SetVideoMode(vid.width, vid.height, BitsPerPixel, surfaceFlags);

    if(NULL == vidSurface)
    {
        CONS_Printf("Could not set vidmode\n");
        return;
    }
    
#ifdef _WIN32
	SDL_VERSION(&WinFo.version);
	if ((WMret = SDL_GetWMInfo(&WinFo)) == 1)
	{
		hIconBig = LoadIcon(GetModuleHandle(NULL), MAKEINTRESOURCE(1));
		hIconSmall = LoadIcon(GetModuleHandle(NULL), MAKEINTRESOURCE(2));
		hWindow = WinFo.window;
#if defined(_MSC_VER) && (_MSC_VER <= 1200)			// VC 6 does NOT have the following
		SetClassLong(hWindow, GCL_HICON, (LONG)hIconBig);
		SetClassLong(hWindow, GCL_HICONSM, (LONG)hIconBig);
#else
		SetClassLongPtr(hWindow, GCLP_HICON, (LONG_PTR) hIconBig);
		SetClassLongPtr(hWindow, GCLP_HICONSM, (LONG_PTR) hIconSmall);
#endif
		CONS_Printf("Set window icon.\n");
	}
	else
		CONS_Printf("Could not set window icon (%i).\n", WMret);
#endif

    vid.buffer = malloc(vid.width * vid.height * vid.bpp * NUMSCREENS);
    vid.direct = vidSurface->pixels; // FIXME

	SDL_ShowCursor(0);

    doUngrabMouse();

    graphics_started = 1;

    return;
}

void I_ShutdownGraphics(void)
{
    // was graphics initialized anyway?
    if (!graphics_started)
        return;
        
    if(NULL != vidSurface)
    {
        SDL_FreeSurface(vidSurface);
        vidSurface = NULL;
    }

    SDL_Quit();

}

