// Emacs style mode select   -*- C++ -*- 
// -----------------------------------------------------------------------------
// ########   ###### #####   #####  ######   ######  ######
// ##     ##  ##     ##  ## ##  ## ##    ## ##    ## ##   ##
// ##     ##  ##     ##   ###   ## ##    ## ##    ## ##    ##
// ########   ####   ##    #    ## ##    ## ##    ## ##    ##
// ##    ##   ##     ##         ## ##    ## ##    ## ##    ##
// ##     ##  ##     ##         ## ##    ## ##    ## ##   ##
// ##      ## ###### ##         ##  ######   ######  ######
//                      http://remood.sourceforge.net/
// -----------------------------------------------------------------------------
// Project Leader:    GhostlyDeath           (ghostlydeath@gmail.com)
// Project Co-Leader: RedZTag                (jostol27@gmail.com)
// Members:           Demyx                  (demyx@endgameftw.com)
// -----------------------------------------------------------------------------
// Copyright (C) 1993-1996 by id Software, Inc.
// Portions Copyright (C) 1998-2000 by DooM Legacy Team.
// Portions Copyright (C) 2008-2009 The ReMooD Team..
// -----------------------------------------------------------------------------
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// -----------------------------------------------------------------------------
// DESCRIPTION:
//      DOOM selection menu, options, episode etc.
//      Sliders and icons. Kinda widget stuff.
//
// NOTE:
//      All V_DrawPatchDirect () has been replaced by V_DrawScaledPatch ()
//      so that the menu is scaled to the screen size. The scaling is always
//      an integer multiple of the original size, so that the graphics look
//      good.

#ifndef _WIN32
#include <unistd.h>
#endif
#include <fcntl.h>

#include "am_map.h"

#include "doomdef.h"
#include "dstrings.h"
#include "d_main.h"

#include "console.h"

#include "r_local.h"
#include "hu_stuff.h"
#include "g_game.h"
#include "g_input.h"

#include "m_argv.h"

// Data.
#include "sounds.h"
#include "s_sound.h"
#include "i_system.h"
#include "i_sound.h"

#include "m_menu.h"
#include "v_video.h"
#include "i_video.h"

#include "keys.h"
#include "z_zone.h"
#include "w_wad.h"
#include "p_local.h"
#include "p_fab.h"
#include "t_script.h"

#include "d_net.h"
#include "p_inter.h"
#include "d_prof.h"

boolean localgame;

// -1 = no quicksave slot picked!
int quickSaveSlot;
boolean menuactive;

// we are going to be entering a savegame string
int saveStringEnter;
int saveSlot;					// which slot to save in
int saveCharIndex;				// which char we're editing
char saveOldString[SAVESTRINGSIZE]; // old save description before edit

char savegamestrings[10][SAVESTRINGSIZE];

// current menudef
menu_t *currentMenu;
short itemOn;					// menu item skull is on
short skullAnimCounter;			// skull animation counter
short whichSkull;				// which skull to draw
int SkullBaseLump;

// graphic name of skulls
char skullName[2][9] = { "M_SKULL1", "M_SKULL2" };

const char *ALLREADYPLAYING = "You are already playing\n\nLeave this game first\n";

menu_t
	MainDef,					// Main Menu
	OptionsDef,					// Options Menu
	DefaultKeyBindDef,			// Key Binds
	HeadsUpDef,					// Heads up Display
	GraphicsDef,				// Graphics
	GameOptionsDef,				// Game Options
	NewGameDef,					// New Game
	NewGameClassicDef,			// New Game -> Classic
	ConsoleDef,					// Console Options
	SoundsDef,					// Sound Options
	VideoDef,					// Video Options
	MiscOptionsDef,				// Options -> Misc Options
	MouseOptionsDef,			// Options -> Mouse
	JoystickOptionsDef,			// Options -> Joysticks
	CreatLocalGameDef,			// New Game -> Local
	NewGameOptionsDef,			// New Game -> Options
	NewGameCCSkillDef,
	NewGameCCEpiDef,
	
	LASTMENU
	;

menu_t* MenuPtrList[] =
{
	&MainDef,
	&OptionsDef,
	&DefaultKeyBindDef,
	&HeadsUpDef,
	&GraphicsDef,
	&GameOptionsDef,
	&NewGameDef,
	&NewGameClassicDef,
	&ProfileDef,
	&ConsoleDef,
	&SoundsDef,
	&VideoDef,
	&MiscOptionsDef,
	&JoystickOptionsDef,
	&MouseOptionsDef,
	&CreatLocalGameDef,
	&NewGameOptionsDef,
	&NewGameCCSkillDef,
	&NewGameCCEpiDef,
	
	NULL,	// To determine the end
};

// =============================================================================
//                                MAIN MENU
// =============================================================================

menuitem_t MainItems[] =
{
	{ITX_MAINMENUITEM | IT_SUBMENU, NULL,		"New Game", &NewGameDef},
	{ITX_MAINMENUITEM | IT_CALL, NULL,			"End Game", M_EndGame},
	{ITX_MAINMENUITEM | IT_DISABLED2, NULL,		"Load Game", NULL},
	{ITX_MAINMENUITEM | IT_DISABLED2, NULL,		"Save Game", NULL},
	{ITX_MAINMENUITEM | IT_SUBMENU,	 NULL,		"Options", &OptionsDef},
	{ITX_MAINMENUITEM | IT_CALL, NULL,			"Profiles", M_StartProfiler},
	{ITX_MAINMENUITEM | IT_CALL, NULL,			"Quit Game", M_QuitDOOM},
};

menu_t MainDef =
{
	MENUFLAG_OPTIMALSPACE,
	"*",
	NULL,
	sizeof(MainItems) / sizeof(menuitem_t),
	MainItems,
	NULL,
	M_DrawGenericMenu,
	0,
	0,
	0,
	0,
	1,
};

// =============================================================================
//                              OPTIONS MENU
// =============================================================================

menuitem_t OptionsItems[] =
{
	{ITX_SUBMENUTITLE, NULL,			"*** GAME ***", NULL},
	{IT_STRING | IT_SUBMENU, NULL,		"Options and Flags...", &GameOptionsDef},
	{IT_STRING | IT_SUBMENU, NULL,		"Heads up Display...", &HeadsUpDef},
	{IT_STRING | IT_SUBMENU, NULL,		"Graphics...", &GraphicsDef},
	{IT_STRING | IT_SUBMENU, NULL,		"Console...", &ConsoleDef},
	{IT_STRING | IT_SUBMENU, NULL,		"Mouse...", &MouseOptionsDef},
	{IT_STRING | IT_SUBMENU, NULL,		"Joysticks...", &JoystickOptionsDef},
	{IT_STRING | IT_SUBMENU, NULL,		"Miscellaneous...", &MiscOptionsDef},
	{ITX_SUBMENUTITLE, NULL,			"*** SYSTEM ***", NULL},
	{IT_STRING | IT_CALL, NULL,			"Video...", &M_StartVideoOptions},
	{IT_STRING | IT_SUBMENU, NULL,		"Sound...", &SoundsDef},
	{ITX_SUBMENUTITLE, NULL,			"*** PLAYERS ***", NULL},
	{IT_STRING | IT_CALL, NULL,			"Default Key Binds (Player 1)...", M_ControlsDoPlayer1},
	{IT_STRING | IT_CALL, NULL,			"Default Key Binds (Player 2)...", M_ControlsDoPlayer2},
	{IT_STRING | IT_CALL, NULL,			"Default Key Binds (Player 3)...", M_ControlsDoPlayer3},
	{IT_STRING | IT_CALL, NULL,			"Default Key Binds (Player 4)...", M_ControlsDoPlayer4},
};

menu_t OptionsDef =
{
	MENUFLAG_OPTIMALSPACE,
	"M_OPTIONS",
	"Options",
	sizeof(OptionsItems) / sizeof(menuitem_t),
	OptionsItems,
	&MainDef,
	M_DrawGenericMenu,
	0,
	0,
	0,
	0,
	1,
};

// =============================================================================
//                           DEFUALT KEY BINDS MENU
// =============================================================================

menuitem_t DefaultKeyBindItems[] =
{
	{ITX_SUBMENUTITLE, NULL,			"*** MOVEMENT ***", NULL},
	{IT_CALL | IT_STRING2, 0, "Fire", M_ChangeControl, gc_fire},
	{IT_CALL | IT_STRING2, 0, "Use/Open", M_ChangeControl, gc_use},
	{IT_CALL | IT_STRING2, 0, "Forward", M_ChangeControl, gc_forward},
	{IT_CALL | IT_STRING2, 0, "Backward", M_ChangeControl, gc_backward},
	{IT_CALL | IT_STRING2, 0, "Turn Left", M_ChangeControl, gc_turnleft},
	{IT_CALL | IT_STRING2, 0, "Turn Right", M_ChangeControl, gc_turnright},
	{IT_CALL | IT_STRING2, 0, "Run", M_ChangeControl, gc_speed},
	{IT_CALL | IT_STRING2, 0, "Strafe On", M_ChangeControl, gc_strafe},
	{IT_CALL | IT_STRING2, 0, "Strafe Left", M_ChangeControl, gc_strafeleft},
	{IT_CALL | IT_STRING2, 0, "Strafe Right", M_ChangeControl, gc_straferight},
	{IT_CALL | IT_STRING2, 0, "Look Up", M_ChangeControl, gc_lookup},
	{IT_CALL | IT_STRING2, 0, "Look Down", M_ChangeControl, gc_lookdown},
	{IT_CALL | IT_STRING2, 0, "Center View", M_ChangeControl, gc_centerview},
	{IT_CALL | IT_STRING2, 0, "Mouselook", M_ChangeControl, gc_mouseaiming},
	{IT_CALL | IT_STRING2, 0, "Jump/Fly Up", M_ChangeControl, gc_jump},
	{IT_CALL | IT_STRING2, 0, "Fly down", M_ChangeControl, gc_flydown},
	{ITX_SUBMENUTITLE, NULL,			"*** WEAPONS & ITEMS ***", NULL},
	{IT_CALL | IT_STRING2, 0, "Fist/Chainsaw/Staff/Gauntlets", M_ChangeControl, gc_weapon1},
	{IT_CALL | IT_STRING2, 0, "Pistol/Wand", M_ChangeControl, gc_weapon2},
	{IT_CALL | IT_STRING2, 0, "Shotgun/Super Shotgun/Crossbow", M_ChangeControl, gc_weapon3},
	{IT_CALL | IT_STRING2, 0, "Chaingun/Dragon Claw", M_ChangeControl, gc_weapon4},
	{IT_CALL | IT_STRING2, 0, "Rocket Launcher/Hellstaff", M_ChangeControl, gc_weapon5},
	{IT_CALL | IT_STRING2, 0, "Plasma rifle/Phoenix Rod", M_ChangeControl, gc_weapon6},
	{IT_CALL | IT_STRING2, 0, "BFG/Firemace", M_ChangeControl, gc_weapon7},
	{IT_CALL | IT_STRING2, 0, "Chainsaw", M_ChangeControl, gc_weapon8},
	{IT_CALL | IT_STRING2, 0, "Previous Weapon", M_ChangeControl, gc_prevweapon},
	{IT_CALL | IT_STRING2, 0, "Next Weapon", M_ChangeControl, gc_nextweapon},
	{IT_CALL | IT_STRING2, 0, "Best Weapon", M_ChangeControl, gc_bestweapon},
	{IT_CALL | IT_STRING2, 0, "Inventory Left", M_ChangeControl, gc_invprev},
	{IT_CALL | IT_STRING2, 0, "Inventory Right", M_ChangeControl, gc_invnext},
	{IT_CALL | IT_STRING2, 0, "Inventory Use", M_ChangeControl, gc_invuse},
	{ITX_SUBMENUTITLE, NULL,			"*** MISC ***", NULL},
	{IT_CALL | IT_STRING2, 0, "Talk key", M_ChangeControl, gc_talkkey},
	{IT_CALL | IT_STRING2, 0, "Rankings/Scores", M_ChangeControl, gc_scores},
	{IT_CALL | IT_STRING2, 0, "Console", M_ChangeControl, gc_console},
};

menu_t DefaultKeyBindDef =
{
	MENUFLAG_OPTIMALSPACE,
	"M_CONTRO",
	"Setup Controls",
	sizeof(DefaultKeyBindItems) / sizeof(menuitem_t),
	DefaultKeyBindItems,
	&OptionsDef,
	M_DrawControl,
	0,
	0,
	0,
	0,
	1
};

// =============================================================================
//                       HEADS UP DISPLAY OPTIONS MENU
// =============================================================================

menuitem_t HeadsUpItems[] =
{
	{IT_STRING | IT_CVAR, 0, "Scale Status Bar", &cv_scalestatusbar, 0},
	{IT_STRING | IT_CVAR, 0, "Transparent Status Bar", &cv_transparentstatusbar, 0},
	{IT_STRING | IT_CVAR, 0, "Status Bar Transparency Level", &cv_transparentstatusbarmode, 0},
	{IT_STRING | IT_CVAR, 0, "Crosshair", &cv_crosshair, 0},
	{IT_STRING | IT_CVAR, 0, "Messages", &cv_showmessages, 0},
};

menu_t HeadsUpDef =
{
	MENUFLAG_OPTIMALSPACE,
	"M_DISOPT",
	"Heads Up Display Options",
	sizeof(HeadsUpItems) / sizeof(menuitem_t),
	HeadsUpItems,
	&OptionsDef,
	M_DrawGenericMenu,
	0,
	0,
	0,
	0,
	1
};

// =============================================================================
//                          GRAPHIC OPTIONS MENU
// =============================================================================

menuitem_t GraphicsItems[] =
{
	{IT_STRING | IT_CVAR, 0, "Screen Link", &cv_screenslink, 0},
	{IT_STRING | IT_CVAR, 0, "Translucency", &cv_translucency, 0},
	{IT_STRING | IT_CVAR, 0, "Splats", &cv_splats, 0},
	{IT_STRING | IT_CVAR, 0, "Max splats", &cv_maxsplats, 0},
	{IT_STRING | IT_CVAR | IT_CV_SLIDER, 0, "Brightness", &cv_usegamma, 0},
	{IT_STRING | IT_CVAR | IT_CV_SLIDER, 0, "Screen Size", &cv_viewsize, 0}
};

menu_t GraphicsDef =
{
	MENUFLAG_OPTIMALSPACE,
	"M_OPTION",
	"Graphic Options",
	sizeof(GraphicsItems) / sizeof(menuitem_t),
	GraphicsItems,
	&OptionsDef,
	M_DrawGenericMenu,
	0,
	0,
	0,
	0,
	1
};

// =============================================================================
//                           MISC OPTIONS MENU
// =============================================================================

menuitem_t MiscOptionsItems[] =
{
	{IT_STRING | IT_CVAR, 0, "Disable Title Screen Demos", &cv_disabledemos, 0},
	{IT_STRING | IT_CVAR, 0, "Controls per key (Input)", &cv_controlperkey, 0},
	{IT_STRING | IT_CVAR, 0, "Blinking cursor rate", &cv_blinkingrate, 0},
};

menu_t MiscOptionsDef =
{
	MENUFLAG_OPTIMALSPACE,
	"M_OPTION",
	"Miscellaneous Options",
	sizeof(MiscOptionsItems) / sizeof(menuitem_t),
	MiscOptionsItems,
	&OptionsDef,
	M_DrawGenericMenu,
	0,
	0,
	0,
	0,
	1
};

// =============================================================================
//                             MOUSE OPTIONS MENU
// =============================================================================

menuitem_t MouseOptionsItems[] =
{
	{IT_STRING | IT_CVAR, 0, "Enable Mouse", &cv_usemouse, 0},
	{IT_STRING | IT_CVAR, 0, "Use Legacy Settings", &cv_m_legacymouse, 0},
	
	{ITX_SUBMENUTITLE, NULL,			"*** LEGACY ***", NULL},
	{IT_STRING | IT_CVAR, 0, "Use mouse to freelook", &cv_alwaysfreelook, 0},
	{IT_STRING | IT_CVAR, 0, "Use mouse to move", &cv_mousemove, 0},
	{IT_STRING | IT_CVAR, 0, "Invert Y-Axis", &cv_invertmouse, 0},
	{IT_STRING | IT_CVAR | IT_CV_SLIDER, 0, "Move/Turn sensitivity", &cv_m_xsensitivity, 0},
	{IT_STRING | IT_CVAR | IT_CV_SLIDER, 0, "Look Up/Down sensitivity", &cv_mlooksens, 0},
	
	{ITX_SUBMENUTITLE, NULL,			"*** NEW ***", NULL},
	{IT_STRING | IT_CVAR | IT_CV_SLIDER, 0, "X-Axis sensitivity", &cv_m_xsensitivity, 0},
	{IT_STRING | IT_CVAR | IT_CV_SLIDER, 0, "Y-Axis sensitivity", &cv_m_ysensitivity, 0},
	{IT_STRING | IT_CVAR, 0, "X-Axis Movement", &cv_m_xaxismode, 0},
	{IT_STRING | IT_CVAR, 0, "Y-Axis Movement", &cv_m_yaxismode, 0},
	{IT_STRING | IT_CVAR, 0, "X-Axis Movement (Secondary)", &cv_m_xaxissecmode, 0},
	{IT_STRING | IT_CVAR, 0, "Y-Axis Movement (Secondary)", &cv_m_yaxissecmode, 0},
	{IT_STRING | IT_CVAR, 0, "Classic ALT Key", &cv_m_classicalt, 0},
};

menu_t MouseOptionsDef =
{
	MENUFLAG_OPTIMALSPACE,
	"M_OPTION",
	"Mouse Options",
	sizeof(MouseOptionsItems) / sizeof(menuitem_t),
	MouseOptionsItems,
	&OptionsDef,
	M_DrawGenericMenu,
	0,
	0,
	0,
	0,
	1
};

// =============================================================================
//                             JOYSTICK OPTIONS MENU
// =============================================================================

menuitem_t JoystickOptionsItems[] =
{
	/*{IT_STRING | IT_CVAR, 0, "Enable Mouse", &cv_usemouse, 0},
	{IT_STRING | IT_CVAR, 0, "Use mouse to freelook", &cv_alwaysfreelook, 0},
	{IT_STRING | IT_CVAR, 0, "Use mouse to move", &cv_mousemove, 0},
	{IT_STRING | IT_CVAR, 0, "Invert mouse", &cv_invertmouse, 0},
	{IT_STRING | IT_CVAR | IT_CV_SLIDER, 0, "Mouse sensitivity", &cv_mousesens, 0},*/
	{IT_STRING | IT_CVAR | IT_CV_SLIDER, 0, "Mouse freelook speed", &cv_mlooksens, 0}
};

menu_t JoystickOptionsDef =
{
	MENUFLAG_OPTIMALSPACE,
	"M_OPTION",
	"Joystick Options",
	sizeof(JoystickOptionsItems) / sizeof(menuitem_t),
	JoystickOptionsItems,
	&OptionsDef,
	M_DrawGenericMenu,
	0,
	0,
	0,
	0,
	1
};

// =============================================================================
//                            CONSOLE OPTIONS MENU
// =============================================================================

menuitem_t ConsoleItems[] =
{
	{IT_STRING | IT_CVAR, 0, "Message Timeout", &cons_msgtimeout, 0},
	{IT_STRING | IT_CVAR, 0, "Speed", &cons_speed, 0},
	{IT_STRING | IT_CVAR, 0, "Height", &cons_height, 0},
	{IT_STRING | IT_CVAR, 0, "Back Picture", &cons_backpic, 0},
};

menu_t ConsoleDef =
{
	MENUFLAG_OPTIMALSPACE,
	"M_OPTION",
	"Console Options",
	sizeof(ConsoleItems) / sizeof(menuitem_t),
	ConsoleItems,
	&OptionsDef,
	M_DrawGenericMenu,
	0,
	0,
	0,
	0,
	1
};

// =============================================================================
//                               SOUND OPTIONS MENU
// =============================================================================

menuitem_t SoundOptionsItems[] =
{
	{ITX_SUBMENUTITLE, NULL,				"*** OUTPUT ***", NULL},
	{IT_STRING | IT_CVAR, 0,				"Sound Output", &cv_snd_output, 0},
	{IT_STRING | IT_CVAR | IT_CV_STRING, 0,	"Sound Device", &cv_snd_device, 0},
	//{IT_STRING | IT_CVAR, 0,				"Music Output", &cv_, 0},
	//{IT_STRING | IT_CVAR | IT_CV_STRING, 0,	"Music Device", &cv_, 0},
	
	{ITX_SUBMENUTITLE, NULL,				"*** QUALITY ***", NULL},
	{IT_STRING | IT_CVAR, 0,				"Speaker Setup", &cv_snd_speakersetup, 0},
	{IT_STRING | IT_CVAR, 0,				"Sound Quality", &cv_snd_soundquality, 0},
	{IT_STRING | IT_CVAR, 0,				"Sound Density", &cv_snd_sounddensity, 0},
	{IT_STRING | IT_CVAR, 0,				"Simulated PC Speaker Waveform", &cv_snd_pcspeakerwave, 0},
	
	{ITX_SUBMENUTITLE, NULL,				"*** VOLUME ***", NULL},
	{IT_STRING | IT_CVAR | IT_CV_SLIDER, 0,	"Sound Volume", &cv_soundvolume, 0},
	//{IT_STRING | IT_CVAR | IT_CV_SLIDER, 0,	"Music Volume", &cv_musicvolume, 0},
	
	{ITX_SUBMENUTITLE, NULL,				"*** MISC SETTINGS ***", NULL},
	{IT_STRING | IT_CVAR, 0,				"Precache Sounds	", &precachesound},
	{IT_STRING | IT_CVAR, 0,				"Random Sound Pitch", &cv_rndsoundpitch, 0},
	{IT_STRING | IT_CVAR, 0,				"Sound Channels", &cv_snd_channels, 0},
	{IT_STRING | IT_CVAR, 0,				"Reserved Sound Channels", &cv_snd_reservedchannels, 0},
	{IT_STRING | IT_CVAR, 0,				"Multi-Threaded Sound", &cv_snd_multithreaded, 0},
	//{IT_STRING | IT_CVAR, 0,				"Multi-Threaded Music", &cv_mus_multithreaded, 0},
	
	{ITX_SUBMENUTITLE, NULL,				"*** RESET ***", NULL},
	{IT_STRING | IT_CALL, 0,				"Reset Sound", M_ResetSound},
	//{IT_STRING | IT_CALL, 0,				"Reset Music"},

	/*{IT_STRING | IT_CVAR | IT_CV_SLIDER, 0, "Sound Volume", &cv_soundvolume, 0},
	{IT_STRING | IT_CVAR | IT_CV_SLIDER, 0, "Music Volume", &cv_musicvolume, 0},
	{IT_STRING | IT_CVAR | IT_CV_SLIDER, 0, "CD Volume", &cd_volume, 0},
	{IT_STRING | IT_CVAR, 0, "Channel Count", &cv_numChannels, 0},
	{IT_STRING | IT_CVAR, 0, "Reverse Stereo", &stereoreverse, 0},*/
	
};

menu_t SoundsDef =
{
	MENUFLAG_OPTIMALSPACE,
	"M_OPTION",
	"Sound Options",
	sizeof(SoundOptionsItems) / sizeof(menuitem_t),
	SoundOptionsItems,
	&OptionsDef,
	M_DrawGenericMenu,
	0,
	0,
	0,
	0,
	1
};

menuitem_t VideoOptionsItems[] =
{
	{IT_STRING | IT_CVAR, 0, 			"Fullscreen", &cv_fullscreen, 0},
	{IT_WHITESTRING | IT_KEYHANDLER,	0,	"Select Mode (Press 'd' to set as default)", &M_HandleVideoKey, 0},
};

menu_t VideoDef =
{
	MENUFLAG_OPTIMALSPACE/* | MENUFLAG_HIDECURSOR*/,
	"M_OPTION",
	"Video Options",
	sizeof(VideoOptionsItems) / sizeof(menuitem_t),
	VideoOptionsItems,
	&OptionsDef,
	M_DrawVideoOptions,
	0,
	0,
	0,
	0,
	1
};

// =============================================================================
//                              GAME OPTIONS MENU
// =============================================================================

menuitem_t GameOptionsItems[] =
{
	{ITX_SUBMENUTITLE, NULL,			"*** MULTIPLAYER ***", NULL},
	{IT_STRING | IT_CVAR | IT_CVARREADONLY, 0, "Deathmatch Type", &cv_deathmatch, 0},
	{IT_STRING | IT_CVAR | IT_CVARREADONLY, 0, "Fraglimit", &cv_fraglimit, 0},
	{IT_STRING | IT_CVAR | IT_CVARREADONLY, 0, "Timelimit", &cv_timelimit, 0},
	
	{ITX_SUBMENUTITLE, NULL,			"*** TEAM ***", NULL},
	{IT_STRING | IT_CVAR | IT_CVARREADONLY, 0, "Teamplay", &cv_teamplay, 0},
	{IT_STRING | IT_CVAR | IT_CVARREADONLY, 0, "TeamDamage", &cv_teamdamage, 0},
	
	{ITX_SUBMENUTITLE, NULL,			"*** RESTRICTIONS ***", NULL},
	{IT_STRING | IT_CVAR | IT_CVARREADONLY, 0, "Allow Jump", &cv_allowjump, 0},
	{IT_STRING | IT_CVAR | IT_CVARREADONLY, 0, "Allow Rocket Jump", &cv_allowrocketjump, 0},
	{IT_STRING | IT_CVAR | IT_CVARREADONLY, 0, "Allow autoaim", &cv_allowautoaim, 0},
	{IT_STRING | IT_CVAR | IT_CVARREADONLY, 0, "Allow turbo", &cv_allowturbo, 0},
	{IT_STRING | IT_CVAR | IT_CVARREADONLY, 0, "Allow exitlevel", &cv_allowexitlevel, 0},
	{IT_STRING | IT_CVAR | IT_CVARREADONLY, 0, "Force Autoaim", &cv_forceautoaim, 0},
	
	{ITX_SUBMENUTITLE, NULL,			"*** WEAPONS & ITEMS ***", NULL},
	{IT_STRING | IT_CVAR | IT_CVARREADONLY, 0, "Item Respawn", &cv_itemrespawn, 0},
	{IT_STRING | IT_CVAR | IT_CVARREADONLY, 0, "Item Respawn time", &cv_itemrespawntime, 0},
	{IT_STRING | IT_CVAR | IT_CVARREADONLY, 0, "Drop Weapons when you die", &cv_fragsweaponfalling, 0},
	{IT_STRING | IT_CVAR | IT_CVARREADONLY, 0, "Infinite Ammo", &cv_infiniteammo, 0},
	
	{ITX_SUBMENUTITLE, NULL,			"*** MONSTERS ***", NULL},
	{IT_STRING | IT_CVAR | IT_CVARREADONLY, 0, "Spawn Monsters", &cv_spawnmonsters, 0},
	{IT_STRING | IT_CVAR | IT_CVARREADONLY, 0, "Monster Respawn", &cv_respawnmonsters, 0},
	{IT_STRING | IT_CVAR | IT_CVARREADONLY, 0, "Monster Respawn time", &cv_respawnmonsterstime, 0},
	{IT_STRING | IT_CVAR | IT_CVARREADONLY, 0, "Fast Monsters", &cv_fastmonsters, 0},
	{IT_STRING | IT_CVAR | IT_CVARREADONLY, 0, "Predicting Monsters", &cv_predictingmonsters, 0},
	
	{ITX_SUBMENUTITLE, NULL,			"*** MISC ***", NULL},
	{IT_STRING | IT_CVAR | IT_CVARREADONLY, 0, "Gravity", &cv_gravity, 0},
	{IT_STRING | IT_CVAR | IT_CVARREADONLY, 0, "Solid corpse", &cv_solidcorpse, 0},
	{IT_STRING | IT_CVAR | IT_CVARREADONLY, 0, "BloodTime", &cv_bloodtime, 0},
	
	{ITX_SUBMENUTITLE, NULL,			"*** COMPATIBILITY ***", NULL},
	{IT_STRING | IT_CVAR | IT_CVARREADONLY, 0, "Classic Blood", &cv_classicblood, 0},
	{IT_STRING | IT_CVAR | IT_CVARREADONLY, 0, "Classic Rocket Explosions", &cv_classicrocketblast, 0},
	{IT_STRING | IT_CVAR | IT_CVARREADONLY, 0, "Classic Monster Melee Range", &cv_classicmeleerange, 0},
	{IT_STRING | IT_CVAR | IT_CVARREADONLY, 0, "Classic Monster Logic", &cv_classicmonsterlogic, 0},
};

menu_t GameOptionsDef =
{
	MENUFLAG_OPTIMALSPACE,
	"M_OPTION",
	"Game Options",
	sizeof(GameOptionsItems) / sizeof(menuitem_t),
	GameOptionsItems,
	&OptionsDef,
	M_DrawGenericMenu,
	0,
	0,
	0,
	0,
	1
};

// =============================================================================
//                                NEW GAME MENU
// =============================================================================

menuitem_t NewGameOptionsItems[] =
{	
	{ITX_SUBMENUTITLE, NULL,			"*** TEAM ***", NULL},
	{IT_STRING | IT_CVAR, 0, "Teamplay", &cv_ng_teamplay, 0},
	{IT_STRING | IT_CVAR, 0, "TeamDamage", &cv_ng_teamdamage, 0},
	
	{ITX_SUBMENUTITLE, NULL,			"*** RESTRICTIONS ***", NULL},
	{IT_STRING | IT_CVAR, 0, "Allow Jump", &cv_ng_allowjump, 0},
	{IT_STRING | IT_CVAR, 0, "Allow Rocket Jump", &cv_ng_allowrocketjump, 0},
	{IT_STRING | IT_CVAR, 0, "Allow autoaim", &cv_ng_allowautoaim, 0},
	{IT_STRING | IT_CVAR, 0, "Allow turbo", &cv_ng_allowturbo, 0},
	{IT_STRING | IT_CVAR, 0, "Allow exitlevel", &cv_ng_allowexitlevel, 0},
	{IT_STRING | IT_CVAR, 0, "Force Autoaim", &cv_ng_forceautoaim, 0},
	
	{ITX_SUBMENUTITLE, NULL,			"*** WEAPONS & ITEMS ***", NULL},
	{IT_STRING | IT_CVAR, 0, "Item Respawn", &cv_ng_itemrespawn, 0},
	{IT_STRING | IT_CVAR, 0, "Item Respawn time", &cv_ng_itemrespawntime, 0},
	{IT_STRING | IT_CVAR, 0, "Drop Weapons when you die", &cv_ng_fragsweaponfalling, 0},
	
	{ITX_SUBMENUTITLE, NULL,			"*** MONSTERS ***", NULL},
	{IT_STRING | IT_CVAR, 0, "Monster Respawn", &cv_ng_respawnmonsters, 0},
	{IT_STRING | IT_CVAR, 0, "Monster Respawn time", &cv_ng_respawnmonsterstime, 0},
	{IT_STRING | IT_CVAR, 0, "Fast Monsters", &cv_ng_fastmonsters, 0},
	{IT_STRING | IT_CVAR, 0, "Predicting Monsters", &cv_ng_predictingmonsters, 0},
	
	{ITX_SUBMENUTITLE, NULL,			"*** MISC ***", NULL},
	{IT_STRING | IT_CVAR, 0, "Gravity", &cv_ng_gravity, 0},
	{IT_STRING | IT_CVAR, 0, "Solid corpse", &cv_ng_solidcorpse, 0},
	{IT_STRING | IT_CVAR, 0, "BloodTime", &cv_ng_bloodtime, 0},
	
	{ITX_SUBMENUTITLE, NULL,			"*** COMPATIBILITY ***", NULL},
	{IT_STRING | IT_CVAR, 0, "Classic Blood", &cv_ng_classicblood, 0},
	{IT_STRING | IT_CVAR, 0, "Classic Rocket Explosions", &cv_ng_classicrocketblast, 0},
	{IT_STRING | IT_CVAR, 0, "Classic Monster Melee Range", &cv_ng_classicmeleerange, 0},
	{IT_STRING | IT_CVAR, 0, "Classic Monster Logic", &cv_ng_classicmonsterlogic, 0},
};

menu_t NewGameOptionsDef =
{
	MENUFLAG_OPTIMALSPACE,
	"M_OPTION",
	"Game Options",
	sizeof(NewGameOptionsItems) / sizeof(menuitem_t),
	NewGameOptionsItems,
	&NewGameDef,
	M_DrawGenericMenu,
	0,
	0,
	0,
	0,
	1
};


menuitem_t NewGameItems[] =
{
	{ITX_SUBMENUTITLE, NULL,			"*** SINGLE PLAYER ***", NULL},
	{IT_STRING | IT_CALL, NULL,			"Classic...", M_DoNewGameClassicClassic},
	{IT_STRING | IT_CALL, NULL,			"Create Game...", M_DoNewGameClassic},
	{IT_STRING | IT_DISABLED2, NULL,	"Quick Start...", NULL},
	
	{ITX_SUBMENUTITLE, NULL,			"*** MULTI PLAYER ***", NULL},
	{IT_STRING | IT_CALL, NULL,			"Split Screen Game...", M_DoNewGameLocal},
	{IT_STRING | IT_DISABLED2, NULL,	"UDP LAN/Internet Game...", NULL},
	{IT_STRING | IT_DISABLED2, NULL,	"TCP LAN/Internet Game...", NULL},
	{IT_STRING | IT_DISABLED2, NULL,	"Modem Game...", NULL},
	{IT_STRING | IT_DISABLED2, NULL,	"Serial/Null-Modem Game...", NULL},
	{IT_STRING | IT_DISABLED2, NULL,	"Fork Game...", NULL},
};

menu_t NewGameDef =
{
	MENUFLAG_OPTIMALSPACE,
	"M_NGAME",
	"New Game",
	sizeof(NewGameItems) / sizeof(menuitem_t),
	NewGameItems,
	&MainDef,
	M_DrawGenericMenu,
	0,
	0,
	0,
	0,
	1
};

// =============================================================================
//                         CREATE GAME (Single) MENU
// =============================================================================

menuitem_t NewGameCCSkillItems[] =
{
	{IT_STRING | IT_CALL | IT_CENTERSTRING, NULL,	"I'm too young to die", M_SelectSkill},
	{IT_STRING | IT_CALL | IT_CENTERSTRING, NULL,	"Hey, not too rough", M_SelectSkill},
	{IT_STRING | IT_CALL | IT_CENTERSTRING, NULL,	"Hurt me plenty", M_SelectSkill},
	{IT_STRING | IT_CALL | IT_CENTERSTRING, NULL,	"Ultra-violence", M_SelectSkill},
	{IT_STRING | IT_CALL | IT_CENTERSTRING, NULL,	"Nightmare!", M_SelectSkill},
};

menuitem_t NewGameCCEpisodeItems[] =
{
	{IT_STRING | IT_CALL | IT_CENTERSTRING, NULL,	"Knee-Deep In The Dead", M_SelectEpisode},
	{IT_STRING | IT_CALL | IT_CENTERSTRING, NULL,	"The Shores of Hell", M_SelectEpisode},
	{IT_STRING | IT_CALL | IT_CENTERSTRING, NULL,	"Inferno", M_SelectEpisode},
	{IT_STRING | IT_CALL | IT_CENTERSTRING, NULL,	"Thy Flesh Consumed", M_SelectEpisode},
	{IT_STRING | IT_CALL | IT_CENTERSTRING, NULL,	"Episode 5", M_SelectEpisode},
	{IT_STRING | IT_CALL | IT_CENTERSTRING, NULL,	"Episode 6", M_SelectEpisode},
};

menu_t NewGameCCSkillDef =
{
	MENUFLAG_OPTIMALSPACE,
	"M_NGAME",
	"Classic Game",
	sizeof(NewGameCCSkillItems) / sizeof(menuitem_t),
	NewGameCCSkillItems,
	&NewGameDef,
	M_DrawGenericMenu,
	0,
	0,
	0,
	0,
	1
};

menu_t NewGameCCEpiDef =
{
	MENUFLAG_OPTIMALSPACE,
	"M_NGAME",
	"Classic Game",
	sizeof(NewGameCCEpisodeItems) / sizeof(menuitem_t),
	NewGameCCEpisodeItems,
	&NewGameDef,
	M_DrawGenericMenu,
	0,
	0,
	0,
	0,
	1
};

menuitem_t NewGameClassicItems[] =
{
	{IT_STRING | IT_CVAR, NULL,			"Level", &cv_ng_map},
	{IT_STRING | IT_CVAR, NULL,			"Skill", &cv_ng_skill},
	{IT_STRING | IT_CVAR, 0, 			"Spawn Monsters", &cv_ng_spawnmonsters, 0},
	{IT_STRING | IT_CVAR, NULL,			"Options", &cv_ng_options},
	{IT_STRING | IT_SPACE, NULL, " ", NULL},
	{IT_STRING | IT_CALL, NULL,			"Setup Options...", M_ClassicGameOptions},
	{IT_STRING | IT_SPACE, NULL, " ", NULL},
	{IT_WHITESTRING | IT_CALL, NULL,	"Start Game!", M_StartClassicGame},
};

menu_t NewGameClassicDef =
{
	MENUFLAG_OPTIMALSPACE,
	"M_NGAME",
	"Create Game",
	sizeof(NewGameClassicItems) / sizeof(menuitem_t),
	NewGameClassicItems,
	&NewGameDef,
	M_DrawGenericMenu,
	0,
	0,
	0,
	0,
	1
};

// =============================================================================
//                         CREATE GAME (Local) MENU
// =============================================================================

menuitem_t CreatLocalGameItems[] =
{
	{IT_STRING | IT_CVAR, NULL,			"Number of Players", &cv_ng_splitscreen},
	{IT_STRING | IT_CVAR, NULL,			"Level", &cv_ng_map},
	{IT_STRING | IT_CVAR, NULL,			"Skill", &cv_ng_skill},
	{IT_STRING | IT_CVAR, 0, 			"Spawn Monsters", &cv_ng_spawnmonsters, 0},
	{IT_STRING | IT_CVAR, 0, 			"Deathmatch Type", &cv_ng_deathmatch, 0},
	{IT_STRING | IT_CVAR, 0,			"Fraglimit", &cv_ng_fraglimit, 0},
	{IT_STRING | IT_CVAR, 0, 			"Timelimit", &cv_ng_timelimit, 0},
	{IT_STRING | IT_CVAR, NULL,			"Options", &cv_ng_options},
	{IT_STRING | IT_SPACE, NULL, " ", NULL},
	{IT_STRING | IT_CALL, NULL,			"Setup Options...", M_LocalGameOptions},
	{IT_STRING | IT_SPACE, NULL, " ", NULL},
	{IT_WHITESTRING | IT_CALL, NULL,	"Start Game!", M_StartLocalGame},
};

menu_t CreatLocalGameDef =
{
	MENUFLAG_OPTIMALSPACE,
	"M_NGAME",
	"Create Local Game",
	sizeof(CreatLocalGameItems) / sizeof(menuitem_t),
	CreatLocalGameItems,
	&NewGameDef,
	M_DrawGenericMenu,
	0,
	0,
	0,
	0,
	1
};

