// Emacs style mode select   -*- C++ -*- 
// -----------------------------------------------------------------------------
// ########   ###### #####   #####  ######   ######  ######
// ##     ##  ##     ##  ## ##  ## ##    ## ##    ## ##   ##
// ##     ##  ##     ##   ###   ## ##    ## ##    ## ##    ##
// ########   ####   ##    #    ## ##    ## ##    ## ##    ##
// ##    ##   ##     ##         ## ##    ## ##    ## ##    ##
// ##     ##  ##     ##         ## ##    ## ##    ## ##   ##
// ##      ## ###### ##         ##  ######   ######  ######
//                      http://remood.sourceforge.net/
// -----------------------------------------------------------------------------
// Project Leader:    GhostlyDeath           (ghostlydeath@gmail.com)
// Project Co-Leader: RedZTag                (jostol27@gmail.com)
// Members:           Demyx                  (demyx@endgameftw.com)
// -----------------------------------------------------------------------------
// Copyright (C) 1993-1996 by id Software, Inc.
// Portions Copyright (C) 1998-2000 by DooM Legacy Team.
// Portions Copyright (C) 2008-2009 The ReMooD Team..
// -----------------------------------------------------------------------------
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// -----------------------------------------------------------------------------
// DESCRIPTION:
//      DOOM selection menu, options, episode etc.
//      Sliders and icons. Kinda widget stuff.
//

#ifndef _WIN32
#include <unistd.h>
#endif
#include <fcntl.h>

#include "am_map.h"

#include "doomdef.h"
#include "dstrings.h"
#include "d_main.h"

#include "console.h"

#include "r_local.h"
#include "hu_stuff.h"
#include "g_game.h"
#include "g_input.h"

#include "m_argv.h"

// Data.
#include "sounds.h"
#include "s_sound.h"
#include "i_system.h"

#include "m_menu.h"
#include "v_video.h"
#include "i_video.h"

#include "keys.h"
#include "z_zone.h"
#include "w_wad.h"
#include "p_local.h"
#include "p_fab.h"
#include "t_script.h"

#include "d_net.h"
#include "p_inter.h"

// Demyx -- Only here temporary, this needs a new home!
CV_PossibleValue_t blinkingrate_cons_t[] = { {1, "1 (No cursor)"}
, {2, "2 (Very Fast)"}
, {3, "3 (Fast)"}
, {4, "4 (Default)"}
, {5, "5 (Slow)"}
, {6, "6 (Slower)"}
, {7, "7 (Very Slow)"}
, {8, "8 (Slowest)"}
, {9, "9 (No blinking)"}
, {0, NULL}
};

consvar_t cv_blinkingrate = {"blinkingrate", "4", CV_SAVE, blinkingrate_cons_t};

//===========================================================================
//Generic Stuffs (more easy to create menus :))
//===========================================================================

void M_DrawMenuTitle(void)
{
	int xtitle = 0;
	int ytitle = 0;
	
	if (FontBBaseLump && currentMenu->menutitle)
	{
		xtitle = (BASEVIDWIDTH - V_TextBWidth(currentMenu->menutitle)) / 2;
		ytitle = (currentMenu->y - V_TextBHeight(currentMenu->menutitle)) / 2;
		if (xtitle < 0)
			xtitle = 0;
		if (ytitle < 0)
			ytitle = 0;

		V_DrawTextB(currentMenu->menutitle, xtitle, ytitle);
	}
	else if (currentMenu->menutitlepic)
	{
		patch_t *p = W_CachePatchName(currentMenu->menutitlepic, PU_CACHE);
		
		if (currentMenu->menutitlex <= -1)
			xtitle = 94;
		else
			xtitle = currentMenu->menutitlex;
			
		if (currentMenu->menutitley <= -1)
			ytitle = 2;
		else
			ytitle = currentMenu->menutitley;

		if (xtitle < 0)
			xtitle = 0;
		if (ytitle < 0)
			ytitle = 0;
			
		V_DrawScaledPatch(xtitle, ytitle, 0, p);
	}
}

void M_DrawGenericMenu(void)
{
	int x, y, i, cursory = 0;
	int fixedx, fixedy;
	int cursorx1;
	int cursorx2;

	// DRAW MENU
	x = currentMenu->x;
	y = currentMenu->y;

	// draw title (or big pic)
	M_DrawMenuTitle();
	
	// Check itemsperpage
	if (!currentMenu->itemsperpage)
		currentMenu->itemsperpage = 500;

	for (i = currentMenu->firstdraw; i < currentMenu->numitems && i < (currentMenu->firstdraw + currentMenu->itemsperpage); i++)
	{
		if (i == itemOn)
			cursory = y;
		switch (currentMenu->menuitems[i].status & IT_DISPLAY)
		{
			case IT_PATCH:
				if (FontBBaseLump && currentMenu->menuitems[i].text)
				{
					V_DrawTextB(currentMenu->menuitems[i].text, x, y);
					y += FONTBHEIGHT - LINEHEIGHT;
				}
				else if (currentMenu->menuitems[i].patch && currentMenu->menuitems[i].patch[0])
					V_DrawScaledPatch(x, y, 0,
									  W_CachePatchName(currentMenu->menuitems[i].patch, PU_CACHE));
			case IT_NOTHING:
			case IT_DYBIGSPACE:
				y += LINEHEIGHT;
				break;
			case IT_BIGSLIDER:
				M_DrawThermo(x, y, (consvar_t *) currentMenu->menuitems[i].itemaction);
				y += LINEHEIGHT;
				break;
			case IT_STRING:
			case IT_WHITESTRING:
				if (currentMenu->menuitems[i].alphaKey)
					y = currentMenu->y + currentMenu->menuitems[i].alphaKey;
				if (i == itemOn)
					cursory = y;
					
				if (currentMenu->menuitems[i].status & IT_CENTERSTRING)
				{
					fixedx = ((currentMenu->width + currentMenu->x) >> 1) -
						(V_StringWidth(currentMenu->menuitems[i].text) >> 1);
						
					if (i == itemOn)
					{
						cursorx1 = fixedx - 10;
						cursorx2 = fixedx + V_StringWidth(currentMenu->menuitems[itemOn].text) + 4;
					}
				}
				else
					fixedx = x;
				fixedy = y;

				if (currentMenu->menuitems[i].status & IT_DISABLED2)
					V_DrawString(fixedx, fixedy, V_GRAYMAP, currentMenu->menuitems[i].text);
				else
				{
					if ((currentMenu->menuitems[i].status & IT_DISPLAY) == IT_STRING)
						V_DrawString(fixedx, fixedy, 0, currentMenu->menuitems[i].text);
					else
						V_DrawString(fixedx, fixedy, V_WHITEMAP, currentMenu->menuitems[i].text);
				}

				// Cvar specific handling
				switch (currentMenu->menuitems[i].status & IT_TYPE)
					case IT_CVAR:
					{
						consvar_t *cv = (consvar_t *) currentMenu->menuitems[i].itemaction;
						switch (currentMenu->menuitems[i].status & IT_CVARTYPE)
						{
							case IT_CV_SLIDER:
								M_DrawSlider((currentMenu->x + currentMenu->width) - SLIDER_WIDTH,
											 y,
											 ((cv->value -
											   cv->PossibleValue[0].value) *
											  100 /
											  (cv->PossibleValue[1].value -
											   cv->PossibleValue[0].value)));
								//y += STRINGHEIGHT;
							case IT_CV_NOPRINT:	// color use this 
								//y += STRINGHEIGHT;
								break;
							case IT_CV_STRING:
								V_DrawFill((currentMenu->x + currentMenu->width) -
									V_StringWidth(cv->string) - (i == itemOn ? V_StringWidth("_") : 0) - 1,
									y - 1, V_StringWidth(cv->string) + (i == itemOn ? V_StringWidth("_") : 0) + 2,
									V_StringHeight("_") + 2, 0);
								V_DrawString(
									(currentMenu->x + currentMenu->width) -
									V_StringWidth(cv->string) - (i == itemOn ? V_StringWidth("_") : 0),
									y/* + 12*/, V_WHITEMAP, cv->string);
								if (skullAnimCounter < 4 && i == itemOn)
									V_DrawCharacter((currentMenu->x + currentMenu->width) -
										V_StringWidth("_"), y, '_' | 0x80);
								//y += STRINGHEIGHT;
								//y += LINEHEIGHT;
								break;
							default:
								V_DrawString(
									((currentMenu->x + currentMenu->width)) - V_StringWidth(cv->string),
									y,
									(currentMenu->menuitems[i].status & IT_DISABLED2 || currentMenu->menuitems[i].status &  IT_CVARREADONLY? V_GRAYMAP : V_WHITEMAP),
									cv->string);
								//V_DrawString(BASEVIDWIDTH - x -
								//			 V_StringWidth(cv->string), y, V_WHITEMAP, cv->string);
								//y += STRINGHEIGHT;
								break;
						}
						break;
					}
				y += STRINGHEIGHT;
				break;
			case IT_STRING2:
				V_DrawString(x, y, 0, currentMenu->menuitems[i].text);
			case IT_DYLITLSPACE:
				y += STRINGHEIGHT;
				break;
			case IT_GRAYPATCH:
				if (currentMenu->menuitems[i].status & IT_PATCH)
				{
					if (FontBBaseLump && currentMenu->menuitems[i].text)
					{
						V_DrawTextBGray(currentMenu->menuitems[i].text, x, y);
						y += FONTBHEIGHT - LINEHEIGHT;
					}
					else if (currentMenu->menuitems[i].patch && currentMenu->menuitems[i].patch[0])
						V_DrawMappedPatch(x, y, 0,
										  W_CachePatchName(currentMenu->
														   menuitems[i].patch, PU_CACHE), graymap);
					y += LINEHEIGHT;
				}
				break;

		}
	}

	// DRAW THE SKULL CURSOR
	if (!(currentMenu->extraflags & MENUFLAG_HIDECURSOR))
	{
		if (((currentMenu->menuitems[itemOn].status & IT_DISPLAY) == IT_PATCH)
			|| ((currentMenu->menuitems[itemOn].status & IT_DISPLAY) == IT_NOTHING))
		{
			V_DrawScaledPatch(currentMenu->x + SKULLXOFF,
							  cursory - 5, 0, W_CachePatchName(skullName[whichSkull], PU_CACHE));
		}
		else if (skullAnimCounter < cv_blinkingrate.value * NEWTICRATERATIO)	//blink cursor
		{
			if (currentMenu->menuitems[itemOn].status & IT_CENTERSTRING)
			{
				V_DrawCharacter(cursorx1, cursory, '*' | 0x80);
				V_DrawCharacter(cursorx2, cursory, '*' | 0x80);
			}
			else
				V_DrawCharacter(currentMenu->x - 10, cursory, '*' | 0x80);
		}
	}
}

//===========================================================================
//                              Some Draw routine
//===========================================================================

//
//      Menu Functions
//
void M_DrawThermo(int x, int y, consvar_t * cv)
{
	int xx, i;
	int leftlump, rightlump, centerlump[2], cursorlump;
	patch_t *p;

	xx = x;
	leftlump = W_GetNumForName("M_THERML");
	rightlump = W_GetNumForName("M_THERMR");
	centerlump[0] = W_GetNumForName("M_THERMM");
	centerlump[1] = W_GetNumForName("M_THERMM");
	cursorlump = W_GetNumForName("M_THERMO");
	V_DrawScaledPatch(xx, y, 0, p = W_CachePatchNum(leftlump, PU_CACHE));
	xx += SHORT(p->width) - SHORT(p->leftoffset);
	for (i = 0; i < 16; i++)
	{
		V_DrawScaledPatch(xx, y, 0, W_CachePatchNum(centerlump[i & 1], PU_CACHE));
		xx += 8;
	}
	V_DrawScaledPatch(xx, y, 0, W_CachePatchNum(rightlump, PU_CACHE));

	xx = (cv->value - cv->PossibleValue[0].value) * (15 * 8) /
		(cv->PossibleValue[1].value - cv->PossibleValue[0].value);

	V_DrawScaledPatch((x + 8) + xx, y, 0, W_CachePatchNum(cursorlump, PU_CACHE));
}

void M_DrawEmptyCell(menu_t * menu, int item)
{
	V_DrawScaledPatch(menu->x - 10, menu->y + item * LINEHEIGHT - 1, 0,
					  W_CachePatchName("M_CELL1", PU_CACHE));
}

void M_DrawSelCell(menu_t * menu, int item)
{
	V_DrawScaledPatch(menu->x - 10, menu->y + item * LINEHEIGHT - 1, 0,
					  W_CachePatchName("M_CELL2", PU_CACHE));
}

//
//  Draw a textbox, like Quake does, because sometimes it's difficult
//  to read the text with all the stuff in the background...
//
//added:06-02-98:
extern int st_borderpatchnum;	//st_stuff.c (for Glide)
void M_DrawTextBox(int x, int y, int width, int lines)
{
	patch_t *p;
	int cx, cy;
	int n;
	int step, boff;

	step = 8;
	boff = 8;

	// draw left side
	cx = x;
	cy = y;
	V_DrawScaledPatch(cx, cy, 0, W_CachePatchNum(viewborderlump[BRDR_TL], PU_CACHE));
	cy += boff;
	p = W_CachePatchNum(viewborderlump[BRDR_L], PU_CACHE);
	for (n = 0; n < lines; n++)
	{
		V_DrawScaledPatch(cx, cy, 0, p);
		cy += step;
	}
	V_DrawScaledPatch(cx, cy, 0, W_CachePatchNum(viewborderlump[BRDR_BL], PU_CACHE));

	// draw middle
	V_DrawFlatFill(x + boff, y + boff, width * step, lines * step, st_borderpatchnum);

	cx += boff;
	cy = y;
	while (width > 0)
	{
		V_DrawScaledPatch(cx, cy, 0, W_CachePatchNum(viewborderlump[BRDR_T], PU_CACHE));

		V_DrawScaledPatch(cx, y + boff + lines * step, 0,
						  W_CachePatchNum(viewborderlump[BRDR_B], PU_CACHE));
		width--;
		cx += step;
	}

	// draw right side
	cy = y;
	V_DrawScaledPatch(cx, cy, 0, W_CachePatchNum(viewborderlump[BRDR_TR], PU_CACHE));
	cy += boff;
	p = W_CachePatchNum(viewborderlump[BRDR_R], PU_CACHE);
	for (n = 0; n < lines; n++)
	{
		V_DrawScaledPatch(cx, cy, 0, p);
		cy += step;
	}
	V_DrawScaledPatch(cx, cy, 0, W_CachePatchNum(viewborderlump[BRDR_BR], PU_CACHE));
}

//==========================================================================
//                        Message is now a (hackeble) Menu
//==========================================================================
void M_DrawMessageMenu(void);

menuitem_t MessageMenu[] = {
	// TO HACK
	{0, NULL, 0, NULL, 0}
};

menu_t MessageDef = {
	0,
	NULL,						// title
	NULL,
	1,							// # of menu items
	MessageMenu,				// menuitem_t ->
	NULL,						// previous menu       (TO HACK)
	M_DrawMessageMenu,			// drawing routine ->
	0, 0,						// x,y                 (TO HACK)
	0							// lastOn, flags       (TO HACK)
};

void M_StartMessage(const char *string, void *routine, menumessagetype_t itemtype)
{
	int max, start, i, lines;
#define message MessageDef.menuitems[0].text
	if (message)
		Z_Free(message);
	message = Z_StrDup(string);
	DEBFILE(message);

	M_StartControlPanel();		// can't put menuactiv to true
	MessageDef.prevMenu = currentMenu;
	MessageDef.menuitems[0].text = message;
	MessageDef.menuitems[0].alphaKey = itemtype;
	switch (itemtype)
	{
		case MM_NOTHING:
			MessageDef.menuitems[0].status = IT_MSGHANDLER;
			MessageDef.menuitems[0].itemaction = M_StopMessage;
			break;
		case MM_YESNO:
			MessageDef.menuitems[0].status = IT_MSGHANDLER;
			MessageDef.menuitems[0].itemaction = routine;
			break;
		case MM_EVENTHANDLER:
			MessageDef.menuitems[0].status = IT_MSGHANDLER;
			MessageDef.menuitems[0].itemaction = routine;
			break;
	}
	//added:06-02-98: now draw a textbox around the message
	// compute lenght max and the numbers of lines
	max = 0;
	start = 0;
	for (lines = 0; *(message + start); lines++)
	{
		for (i = 0; i < (int)strlen(message + start); i++)
		{
			if (*(message + start + i) == '\n')
			{
				if (i > max)
					max = i;
				start += i + 1;
				i = -1;			//added:07-02-98:damned!
				break;
			}
		}

		if (i == (int)strlen(message + start))
			start += i;
	}

	MessageDef.x = (BASEVIDWIDTH - 8 * max - 16) / 2;
	MessageDef.y = (BASEVIDHEIGHT - M_StringHeight(message)) / 2;

	MessageDef.lastOn = (lines << 8) + max;

//    M_SetupNextMenu();
	currentMenu = &MessageDef;
	itemOn = 0;
}

#define MAXMSGLINELEN 256

void M_DrawMessageMenu(void)
{
	int y;
	short i, max;
	char string[MAXMSGLINELEN];
	int start, lines;
	char *msg = currentMenu->menuitems[0].text;

	y = currentMenu->y;
	start = 0;
	lines = currentMenu->lastOn >> 8;
	max = (currentMenu->lastOn & 0xFF) * 8;
	M_DrawTextBox(currentMenu->x, y - 8, (max + 7) >> 3, lines);

	while (*(msg + start))
	{
		for (i = 0; i < (int)strlen(msg + start); i++)
		{
			if (*(msg + start + i) == '\n')
			{
				memset(string, 0, MAXMSGLINELEN);
				if (i >= MAXMSGLINELEN)
				{
					CONS_Printf("M_DrawMessageMenu: too long segment in %s\n", msg);
					return;
				}
				else
				{
					strncpy(string, msg + start, i);
					start += i + 1;
					i = -1;		//added:07-02-98:damned!
				}

				break;
			}
		}

		if (i == (int)strlen(msg + start))
		{
			if (i >= MAXMSGLINELEN)
			{
				CONS_Printf("M_DrawMessageMenu: too long segment in %s\n", msg);
				return;
			}
			else
			{
				strcpy(string, msg + start);
				start += i;
			}
		}

		V_DrawString((BASEVIDWIDTH - V_StringWidth(string)) / 2, y, 0, string);
		y += 8;					//SHORT(hu_font[0]->height);
	}
}

// default message handler
void M_StopMessage(int choice)
{
	M_SetupNextMenu(MessageDef.prevMenu);
	S_StartSound(NULL, sfx_swtchx);
}

//==========================================================================
//                        Menu stuffs
//==========================================================================

//added:30-01-98:
//
//  Write a string centered using the hu_font
//
void M_CentreText(int y, char *string)
{
	int x;
	//added:02-02-98:centre on 320, because V_DrawString centers on vid.width...
	x = (BASEVIDWIDTH - V_StringWidth(string)) >> 1;
	V_DrawString(x, y, 0, string);
}

//
//      Find string height from hu_font chars
//
int M_StringHeight(char *string)
{
	int i;
	int h;
	int height = 8;				//(hu_font[0]->height);

	h = height;
	for (i = 0; i < (int)strlen(string); i++)
		if (string[i] == '\n')
			h += height;

	return h;
}

//
// M_Drawer
// Called after the view has been rendered,
// but before it has been blitted.
//
void M_Drawer(void)
{
	if (!menuactive)
		return;

	//added:18-02-98:
	// center the scaled graphics for the menu,
	//  set it 0 again before return!!!
	//scaledofs = vid.centerofs;

	// now that's more readable with a faded background (yeah like Quake...)
	V_DrawFadeScreen();

	if (currentMenu->drawroutine)
		currentMenu->drawroutine();	// call current menu Draw routine

	//added:18-02-98: it should always be 0 for non-menu scaled graphics.
	scaledofs = 0;

}

//
// M_StartControlPanel
//
void M_StartControlPanel(void)
{
	// intro might call this repeatedly
	if (menuactive)
		return;

	menuactive = 1;
	currentMenu = &MainDef;		// JDC
	itemOn = currentMenu->lastOn;	// JDC

	CON_ToggleOff();			// dirty hack : move away console
}

//
// M_ClearMenus
//
void M_ClearMenus(boolean callexitmenufunc)
{
	if (!menuactive)
		return;

	if (currentMenu->quitroutine && callexitmenufunc)
	{
		if (!currentMenu->quitroutine())
			return;				// we can't quit this menu (also used to set parameter from the menu)
	}

	menuactive = 0;
}

//
//  A smaller 'Thermo', with range given as percents (0-100)
//
void M_DrawSlider(int x, int y, int range)
{
	int i;

	if (range < 0)
		range = 0;
	if (range > 100)
		range = 100;

	V_DrawScaledPatch(x - 8, y, 0, W_CachePatchName("M_SLIDEL", PU_CACHE));

	for (i = 0; i < SLIDER_RANGE; i++)
		V_DrawScaledPatch(x + i * 8, y, 0, W_CachePatchName("M_SLIDEM", PU_CACHE));

	V_DrawScaledPatch(x + SLIDER_RANGE * 8, y, 0, W_CachePatchName("M_SLIDER", PU_CACHE));

	// draw the slider cursor
	V_DrawScaledPatch(x + ((SLIDER_RANGE - 1) * 8 * range) / 100, y, 0,
					  W_CachePatchName("M_SLIDEC", PU_CACHE));
}

int (*setupcontrols)[2] = gamecontrol;

//
//  Draws the Customise Controls menu
//
void M_DrawControl(void)
{
	char tmp[50];
	int i;
	int keys[2];

	// draw title, strings and submenu
	M_DrawGenericMenu();

	for (i = currentMenu->firstdraw; i < currentMenu->numitems && i < (currentMenu->firstdraw + currentMenu->itemsperpage); i++)
	{
		if (currentMenu->menuitems[i].status != IT_CONTROL)
			continue;

		keys[0] = setupcontrols[currentMenu->menuitems[i].alphaKey][0];
		keys[1] = setupcontrols[currentMenu->menuitems[i].alphaKey][1];

		tmp[0] = '\0';
		if (keys[0] == KEY_NULL && keys[1] == KEY_NULL)
		{
			strcpy(tmp, "---");
		}
		else
		{
			if (keys[0] != KEY_NULL)
				strcat(tmp, G_KeynumToString(keys[0]));

			if (keys[0] != KEY_NULL && keys[1] != KEY_NULL)
				strcat(tmp, " or ");

			if (keys[1] != KEY_NULL)
				strcat(tmp, G_KeynumToString(keys[1]));

		}
		V_DrawString((DefaultKeyBindDef.x + DefaultKeyBindDef.width) - V_StringWidth(tmp),
					 DefaultKeyBindDef.y + (i - currentMenu->firstdraw) * STRINGHEIGHT, V_WHITEMAP, tmp);
	}
}


