// Emacs style mode select   -*- C++ -*-
// -----------------------------------------------------------------------------
// ########   ###### #####   #####  ######   ######  ######
// ##     ##  ##     ##  ## ##  ## ##    ## ##    ## ##   ##
// ##     ##  ##     ##   ###   ## ##    ## ##    ## ##    ##
// ########   ####   ##    #    ## ##    ## ##    ## ##    ##
// ##    ##   ##     ##         ## ##    ## ##    ## ##    ##
// ##     ##  ##     ##         ## ##    ## ##    ## ##   ##
// ##      ## ###### ##         ##  ######   ######  ######
//                      http://remood.sourceforge.net/
// -----------------------------------------------------------------------------
// Project Leader:    GhostlyDeath           (ghostlydeath@gmail.com)
// Project Co-Leader: RedZTag                (jostol27@gmail.com)
// Members:           Demyx                  (demyx@endgameftw.com)
// -----------------------------------------------------------------------------
// Copyright (C) 2008-2009 The ReMooD Team.
// -----------------------------------------------------------------------------
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// -----------------------------------------------------------------------------
// DESCRIPTION: Master Server

#ifndef __GLOBAL_H__
#define __GLOBAL_H__

/**** Fixed Size Types ****/
#if defined(_MSC_VER)
#include <time.h>
typedef signed __int8 Int8;
typedef signed __int16 Int16;
typedef signed __int32 Int32;
typedef unsigned __int8 UInt8;
typedef unsigned __int16 UInt16;
typedef unsigned __int32 UInt32;
typedef time_t Time;
#elif defined(__GNUC__) || defined(__GNUG__) || defined(_GNUC_) || defined(_GNUG_)
#if defined(__palmos__)
#include <PalmOS.h>
typedef UInt32 size_t;
typedef UInt32 Time;
#else
#include <stdlib.h>
#include <stdint.h>
#include <time.h>
typedef int8_t Int8;
typedef int16_t Int16;
typedef int32_t Int32;
typedef uint8_t UInt8;
typedef uint16_t UInt16;
typedef uint32_t UInt32;
typedef time_t Time;
#endif
#endif

#if defined(__m68k__) || defined(M68000) || (defined(CPU_TYPE) && defined(CPU_68K) && CPU_TYPE == CPU_68K) || defined(__MC68K__) || defined(__powerpc__) || defined(_M_PPC) || defined(_ARCH_PPC)
#define BIGENDIAN
#define NATIVE16(x) (((x) >> 8) | ((x) << 8))
#define NATIVE32(x) (((x) >> 16) | ((x) << 16))
#else
#define LITTLEENDIAN
#define NATIVE16(x) (x)
#define NATIVE32(x) (x)
#endif

/**** Network Control ****/
typedef union IP_s
{
	UInt32 dw;
	UInt16 w[2];
	UInt8 b[4];
} IP_t;

typedef struct Host_s
{
	IP_t ip;
	UInt16 port;
} Host_t;

typedef enum
{
	NE_SUCCESS,				// Worked
	
	NE_NOTIMPLEMENTED,		// Not Implemented
	NE_BINDTOPORTERROR,		// The port could not be bound
	NE_BUFFERTOOSMALL,		// The buffer is too small
	NE_BUFFEREMPTY,			// The buffer is empty
	NE_SOCKETFAILURE,		// Socket failed to be created
	NE_INVALIDPARAMETER,	// Invalid Parameter
	
	MAXNETWORKERROR
} NetworkError_t;

NetworkError_t NET_BindPort(UInt16 Port);										// Binds a UDP Port
NetworkError_t NET_ClosePort(void);												// Closes the UDP Port
NetworkError_t NET_RecvData(Host_t* From, void* BufferPtr, size_t BufferSize, size_t* RecvSize);
	// Recieves UDP Data
NetworkError_t NET_SendData(Host_t* To, void* BufferPtr, size_t BufferSize, size_t* SentSize);
	// Sends UDP Data
	
Int8 NET_ReadInt8(Int8** Ptr);
Int16 NET_ReadInt16(Int16** Ptr);
Int32 NET_ReadInt32(Int32** Ptr);
UInt8 NET_ReadUInt8(UInt8** Ptr);
UInt16 NET_ReadUInt16(UInt16** Ptr);
UInt32 NET_ReadUInt32(UInt32** Ptr);
void NET_WriteInt8(Int8** Ptr, Int8 Val);
void NET_WriteInt16(Int16** Ptr, Int16 Val);
void NET_WriteInt32(Int32** Ptr, Int32 Val);
void NET_WriteUInt8(UInt8** Ptr, UInt8 Val);
void NET_WriteUInt16(UInt16** Ptr, UInt16 Val);
void NET_WriteUInt32(UInt32** Ptr, UInt32 Val);

/**** Master Server ****/
typedef enum
{
	ME_SUCCESS,				// Worked
	
	ME_NOTIMPLEMENTED,		// Not Implemented
	ME_NOTHINGHAPPENED,		// Nothing happened
	ME_OUTOFMEMORY,			// Out of memory
	ME_OFFLINE,				// Master is offline
	ME_SHUTDOWN,			// Master wants to shutdown
	ME_CRITICALERROR,		// Something bad happened
	ME_STARTING,			// Master server is starting
	
	MAXMASTERERROR
} MasterError_t;

MasterError_t M_MasterMessage(char* Message, ...);								// Print a message from the master
MasterError_t M_StartMaster(void);												// Starts the master server
MasterError_t M_StopMaster(void);												// Stops the master server
MasterError_t M_Status(void);													// Status of the master server
MasterError_t M_Iterate(void);													// Does actions in queue

/**** System Handlers ****/
#if defined(__palmos__)

#include <MemoryMgr.h>
#include <MemGlue.h>
#include <TimeMgr.h>

void* SYS_MemCpy(void* dest, void* src, size_t len);	// Palm OS does not have this! :(
#define SYS_MemSet(dest,val,len) MemSet(dest, len, val)
#define SYS_Malloc(len) MemGluePtrNew(len)
#define SYS_Free(ptr) MemPtrFree(ptr)
#define SYS_SPrintF StrPrintF
#define SYS_StrCaseCmp(a,b) StrCaselessCompare(a, b)
#define SYS_GetTime() TimGetSeconds()

#else

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define SYS_MemCpy(dest,src,len) memcpy(dest, src, len)
#define SYS_MemSet(dest,val,len) memset(dest, val, len)
#define SYS_Malloc(len) malloc(len)
#define SYS_Free(ptr) free(ptr)
#define SYS_SPrintF sprintf
#define SYS_GetTime() time(NULL)
#if defined(_MSC_VER)
#define SYS_StrCaseCmp(a,b) stricmp(a, b)
#else
#define SYS_StrCaseCmp(a,b) strcasecmp(a, b)
#endif

#endif

/*** Protocol Related ***/
typedef enum
{
	PE_SUCCESS,				// Worked
	
	PE_NOTIMPLEMENTED,		// Not Implemented
	PE_NOTHINGHAPPENED,		// Nothing happened
	PE_OUTOFMEMORY,			// Out of memory
	PE_BADPARAMETER,		// Sent a bad parameter
	
	MAXPROTOCOLERROR
} ProtocolError_t;

typedef struct ProtocolHandler_s
{
	UInt32 Code;
	ProtocolError_t (*Func)(Host_t Host, UInt8* InData, size_t InSize, UInt8** OutData, size_t* OutSize);
} ProtocolHandler_t;

extern ProtocolHandler_t MasterProtocols[];
extern size_t NumMasterProtocols;
extern ProtocolHandler_t RMDSevenProtocols[];
extern size_t NumRMDSevenProtocols;

#define CODEIZE(a,b,c,d) ((((UInt8)((a) & 0xFF)) << 24) | (((UInt8)((b) & 0xFF)) << 16) | (((UInt8)((c) & 0xFF)) << 8) | ((UInt8)((d) & 0xFF)))

#define NETWORKBUFFERSIZE 65535

#endif /* __GLOBAL_H__ */

