// Emacs style mode select   -*- C++ -*-
// -----------------------------------------------------------------------------
// ########   ###### #####   #####  ######   ######  ######
// ##     ##  ##     ##  ## ##  ## ##    ## ##    ## ##   ##
// ##     ##  ##     ##   ###   ## ##    ## ##    ## ##    ##
// ########   ####   ##    #    ## ##    ## ##    ## ##    ##
// ##    ##   ##     ##         ## ##    ## ##    ## ##    ##
// ##     ##  ##     ##         ## ##    ## ##    ## ##   ##
// ##      ## ###### ##         ##  ######   ######  ######
//                      http://remood.sourceforge.net/
// -----------------------------------------------------------------------------
// Project Leader:    GhostlyDeath           (ghostlydeath@gmail.com)
// Project Co-Leader: RedZTag                (jostol27@gmail.com)
// Members:           Demyx                  (demyx@endgameftw.com)
// -----------------------------------------------------------------------------
// Copyright (C) 2008-2009 The ReMooD Team.
// -----------------------------------------------------------------------------
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// -----------------------------------------------------------------------------
// DESCRIPTION: Master Server

#include "global.h"

Int8 Control = 0;
MasterError_t Status = ME_OFFLINE;
UInt16 Port = 60100;

/* M_StartMaster() -- Starts the master server */
MasterError_t M_StartMaster(void)
{
	NetworkError_t NetError = NE_SUCCESS;
	
	// Check Status
	if (Status != ME_OFFLINE)
	{
		M_MasterMessage("Master is not offline!");
		return ME_CRITICALERROR;
	}
	
	// Start Everything
	M_MasterMessage("Starting the master server...");
	Status = ME_STARTING;
	
	M_MasterMessage("Attempting to bind to port %i...", Port);
	NetError = NET_BindPort(Port);
	if (NetError)
	{
		if (NetError == NE_BINDTOPORTERROR)
			M_MasterMessage("Bind to port error (port is in use).");
		else
			M_MasterMessage("Unknown failure (returned %i).", NetError);
		return ME_CRITICALERROR;
	}
	
	M_MasterMessage("Port successfully bound!");
	
	Status = ME_SUCCESS;
	
	return ME_SUCCESS;
}

/* M_StopMaster() -- Stops the master server */
MasterError_t M_StopMaster(void)
{
	size_t i = 0;
	M_MasterMessage("Stopping the master server...");
	
	// Control Lock
	if (Control)
	{
		M_MasterMessage("Waiting for control to be relinquished...");
		while (Control);
	}
	
	Control = 1;
	
	// Check Status
	if (Status == ME_SHUTDOWN || Status == ME_OFFLINE)
	{
		if (Status == ME_SHUTDOWN)
			M_MasterMessage("Master is already shutting down!");
		else
			M_MasterMessage("Master is already offline!");
			
		return ME_CRITICALERROR;
	}
	
	Status = ME_SHUTDOWN;
	
	// Stop everything and free up all the used memory
	NET_ClosePort();

	for (i = 0; i < 256; i++)
	{
	}
	
	// Unlock
	Status = ME_OFFLINE;
	Control = 0;
	
	return ME_SUCCESS;
}

/* M_Status() -- Status of the master server */
MasterError_t M_Status(void)
{
	return Status;
}

/* M_IPString() -- IP to a string */
char* M_IPString(Host_t Host)
{
	static char ip[22];	// "xxx.xxx.xxx.xxx:xxxxx"
	
	SYS_SPrintF(ip, "%3i.%3i.%3i.%3i:%5i",
		Host.ip.b[3],
		Host.ip.b[2],
		Host.ip.b[1],
		Host.ip.b[0],
		Host.port);
		
	return ip;
}

/* M_Iterate() -- Does actions in queue */
MasterError_t M_Iterate(void)
{
	static UInt8* Buffer = NULL;
	static Host_t Flow;
	static size_t RecvSize;
	static size_t SentSize;
	static size_t i, j;
	static UInt8* Ptr;
	static UInt32 InCode;
	static UInt32 InSize;
	
	static UInt8* InData = NULL;
	static size_t InDataSize = 0;
	static UInt8* OutData = NULL;
	static size_t OutDataSize = 0;
	static UInt8* OutData2 = NULL;
	static size_t OutDataSize2 = 0;
	static UInt32* CodePtr = NULL;
	static UInt32* SizePtr = NULL;
	
	static Int8 Activity = 0;
	static UInt8* Temp = NULL;
	
	if (!Buffer)
	{
		Buffer = SYS_Malloc(NETWORKBUFFERSIZE);
		SYS_MemSet(Buffer, 0, NETWORKBUFFERSIZE);
	}
		
	if (!Buffer)
	{
		SYS_Free(Buffer);
		Buffer = NULL;
		
		return ME_OUTOFMEMORY;
	}
	
	if (Status == ME_SUCCESS)
	{
		/* Lock */
		if (Control)
		{
			M_MasterMessage("Waiting for control to be relinquished...");
			while (Control);
		}
	
		Control = 1;
		
		/* Do our stuff! */
		// Check Timers and modify the data if needed
		//M_RunAllTimers();
		
		// First get the data (if any) and do it all in a loop for speed
		RecvSize = 0;
		while (!NET_RecvData(&Flow, Buffer, NETWORKBUFFERSIZE, &RecvSize) && RecvSize > 0)
		{	
			if (RecvSize < 8)
				continue;
				
			Ptr = Buffer;
			
			while (Ptr < Buffer + RecvSize)
			{
				InCode = NET_ReadUInt32(Ptr);
				InSize = NET_ReadUInt32(Ptr);
				
				// Check the InSize against the RecvSize
				if (Ptr + InSize > Buffer + RecvSize)
					break;
				
				// Do the required actio
				for (i = 0; i < NumMasterProtocols; i++)
					if (InCode == MasterProtocols[i].Code)
					{
						// Fall out if there is no handler function
						if (!MasterProtocols[i].Func)
							break;
						
						// Send in the INNER Data
						if (!InData)
						{
							InData = SYS_Malloc(InSize);
							InDataSize = InSize;
						}
						else if (InDataSize != InSize)
						{
							SYS_Free(InData);
							InData = SYS_Malloc(InSize);
							InDataSize = InSize;
						}
						
						if (!InData)
							return ME_OUTOFMEMORY;
						
						SYS_MemSet(InData, 0, InDataSize);
						SYS_MemCpy(InData, Ptr, InDataSize);
						Ptr += InDataSize;
						
						// Call Handler
						MasterProtocols[i].Func(Flow, InData, InDataSize, &OutData2, &OutDataSize2);
						
						if (OutData2 && OutDataSize2 != 0)
						{
							if (OutData)
							{
								Temp = SYS_Malloc(OutDataSize + 8 + OutDataSize2);
								SYS_MemSet(Temp, 0, OutDataSize + 8 + OutDataSize2);
								SYS_MemCpy(Temp, OutData, OutDataSize);
								CodePtr = OutData + OutDataSize;
								SizePtr = CodePtr + 4;
								OutDataSize = OutDataSize + 8 + OutDataSize2;
								OutData = Temp;
							}
							else
							{
								OutData = SYS_Malloc(8 + OutDataSize2);
								SYS_MemSet(OutData, 0, 8 + OutDataSize2);
								OutDataSize = 8 + OutDataSize2;
								CodePtr = OutData;
								SizePtr = CodePtr + 4;
							}
				
							*CodePtr = InCode;
							*SizePtr = OutDataSize;
							SYS_MemCpy(CodePtr + 8, OutData2, OutDataSize2);
				
							SYS_Free(OutData2);
							OutData2 = NULL;
							OutDataSize2 = NULL;
						}
						
						// Fly out of the loop
						SYS_Free(InData);
						InData = NULL;
						InDataSize = 0;
						break;
					}
				
				if (i == NumMasterProtocols)
					M_MasterMessage("%s sent an invalid query (%c%c%c%c).", M_IPString(Flow),
						(((((char*)&InCode)[0]) >= 0x20 && (((char*)&InCode)[0]) <= 0x7F) ? (((char*)&InCode)[0]) : '?'),
						(((((char*)&InCode)[1]) >= 0x20 && (((char*)&InCode)[1]) <= 0x7F) ? (((char*)&InCode)[1]) : '?'),
						(((((char*)&InCode)[2]) >= 0x20 && (((char*)&InCode)[2]) <= 0x7F) ? (((char*)&InCode)[2]) : '?'),
						(((((char*)&InCode)[3]) >= 0x20 && (((char*)&InCode)[3]) <= 0x7F) ? (((char*)&InCode)[3]) : '?')
						);
			}
			
			// Send Any Data if it needs to be sent
			if (OutData && OutDataSize >= 8)
			{
				NET_SendData(&Flow, OutData, OutDataSize, &SentSize);
				
				SYS_Free(OutData);
				OutDataSize = 0;
			}
			
			// Clear out
			RecvSize = 0;
			SYS_MemSet(Buffer, 0, NETWORKBUFFERSIZE);
			SYS_MemSet(&Flow, 0, sizeof(Flow));
		}
		
		/* Unlock */
		Control = 0;
	}
	
	if (!Activity)
		return ME_NOTHINGHAPPENED;
	else
		return ME_SUCCESS;
}

