// Emacs style mode select   -*- C++ -*-
// -----------------------------------------------------------------------------
// ########   ###### #####   #####  ######   ######  ######
// ##     ##  ##     ##  ## ##  ## ##    ## ##    ## ##   ##
// ##     ##  ##     ##   ###   ## ##    ## ##    ## ##    ##
// ########   ####   ##    #    ## ##    ## ##    ## ##    ##
// ##    ##   ##     ##         ## ##    ## ##    ## ##    ##
// ##     ##  ##     ##         ## ##    ## ##    ## ##   ##
// ##      ## ###### ##         ##  ######   ######  ######
//                      http://remood.sourceforge.net/
// -----------------------------------------------------------------------------
// Project Leader:    GhostlyDeath           (ghostlydeath@gmail.com)
// Project Co-Leader: RedZTag                (jostol27@gmail.com)
// Members:           Demyx                  (demyx@endgameftw.com)
// -----------------------------------------------------------------------------
// Copyright (C) 2008-2009 The ReMooD Team.
// -----------------------------------------------------------------------------
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// -----------------------------------------------------------------------------
// DESCRIPTION: Master Server

#include "global.h"

ProtocolError_t RMD7(Host_t Host, UInt8* InData, size_t InSize, UInt8** OutData, size_t* OutSize)
{
	static size_t i;
	static UInt8* Buffer;
	
	static UInt8* InData2 = NULL;
	static size_t InDataSize2 = 0;
	static UInt8* OutData2 = NULL;
	static size_t OutDataSize2 = 0;
	static UInt8* Temp = NULL;
	static UInt8* Ptr = NULL;
	
	static UInt32 InCode;
	static size_t InSize2;
	static UInt32* CodePtr = NULL;
	static UInt32* SizePtr = NULL;
	
	if (InSize < 8)
		return PE_BADPARAMETER;
		
	Ptr = InData;
	
	while (Ptr < Buffer + InSize)
	{
		InCode = NET_ReadUInt32(Ptr);
		InSize2 = NET_ReadUInt32(Ptr);
		
		// Check the InSize against the RecvSize
		if (Ptr + InSize2 > Buffer + InSize)
			break;
		
		// Do the required actio
		for (i = 0; i < NumMasterProtocols; i++)
			if (InCode == RMDSevenProtocols[i].Code)
			{
				// Fall out if there is no handler function
				if (!RMDSevenProtocols[i].Func)
					break;
				
				// Send in the INNER Data
				if (!InData2)
				{
					InData2 = SYS_Malloc(InSize2);
					InDataSize2 = InSize2;
				}
				else if (InDataSize2 != InSize2)
				{
					SYS_Free(InData2);
					InData2 = SYS_Malloc(InSize2);
					InDataSize2 = InSize2;
				}
				
				if (!InData)
					return ME_OUTOFMEMORY;
				
				SYS_MemSet(InData2, 0, InDataSize2);
				SYS_MemCpy(InData2, Ptr, InDataSize2);
				Ptr += InDataSize2;
				
				// Call Handler
				RMDSevenProtocols[i].Func(Host, InData2, InDataSize2, &OutData2, &OutDataSize2);
				
				if (OutData2 && OutDataSize2 != 0)
				{
					if (*OutData)
					{
						Temp = SYS_Malloc(*OutSize + 8 + OutDataSize2);
						SYS_MemSet(Temp, 0, *OutSize + 8 + OutDataSize2);
						SYS_MemCpy(Temp, *OutData, *OutSize);
						CodePtr = *OutData + *OutSize;
						SizePtr = CodePtr + 4;
						*OutSize = *OutSize + 8 + OutDataSize2;
						*OutData = Temp;
					}
					else
					{
						*OutData = SYS_Malloc(8 + OutDataSize2);
						SYS_MemSet(*OutData, 0, 8 + OutDataSize2);
						*OutSize = 8 + OutDataSize2;
						CodePtr = *OutData;
						SizePtr = CodePtr + 4;
					}
					
					*CodePtr = InCode;
					*SizePtr = OutDataSize2;
					SYS_MemCpy(CodePtr + 8, OutData2, OutDataSize2);
					
					SYS_Free(OutData2);
					OutData2 = NULL;
					OutDataSize2 = NULL;
				}
				
				// Fly out of the loop
				SYS_Free(InData2);
				InData2 = NULL;
				InDataSize2 = 0;
				break;
			}
		
		if (i == NumMasterProtocols)
			M_MasterMessage("%s sent an invalid query (RMD7:%c%c%c%c).", M_IPString(Host),
				(((((char*)&InCode)[0]) >= 0x20 && (((char*)&InCode)[0]) <= 0x7F) ? (((char*)&InCode)[0]) : '?'),
				(((((char*)&InCode)[1]) >= 0x20 && (((char*)&InCode)[1]) <= 0x7F) ? (((char*)&InCode)[1]) : '?'),
				(((((char*)&InCode)[2]) >= 0x20 && (((char*)&InCode)[2]) <= 0x7F) ? (((char*)&InCode)[2]) : '?'),
				(((((char*)&InCode)[3]) >= 0x20 && (((char*)&InCode)[3]) <= 0x7F) ? (((char*)&InCode)[3]) : '?')
				);
	}
	
	return PE_SUCCESS;
}

/*******************************************************************************
*******************************************************************************/

ProtocolHandler_t MasterProtocols[] =		/* Remember to sort via Priority! */
{
	{CODEIZE('S', 'U', 'P', 'R'), NULL},	// Supported Protocols
	{CODEIZE('R', 'M', 'D', '7'), RMD7},	// ReMooD 0.7
};

size_t NumMasterProtocols = sizeof(MasterProtocols) / sizeof(ProtocolHandler_t);

ProtocolHandler_t RMDSevenProtocols[] =	/* Remember to sort via Priority! */
{
	{CODEIZE('S', 'U', 'P', 'R'), NULL},	// Supported Commands
	{CODEIZE('L', 'S', 'T', '4'), NULL},	// Request: IPv4 Server List (Just Server IPs)
	{CODEIZE('D', 'T', 'L', '4'), NULL},	// Request: IPv4 Server List w/ Details
	{CODEIZE('L', 'S', 'T', '6'), NULL},	// Request: IPv6 Server List (Just Server IPs)
	{CODEIZE('D', 'T', 'L', '6'), NULL},	// Request: IPv6 Server List w/ Details
	{CODEIZE('I', 'N', 'F', 'O'), NULL},	// Request: Server Info (Country, OS, etc.)
	{CODEIZE('P', 'I', 'N', 'G'), NULL},	// Request: Ping
	{CODEIZE('P', 'O', 'N', 'G'), NULL},	// Request: Pong
	{CODEIZE('A', 'D', 'V', 'R'), NULL},	// Request: Advertise
	{CODEIZE('L', 'I', 'N', 'K'), NULL},	// Master Server Link
};

size_t NumRMDSevenProtocols = sizeof(RMDSevenProtocols) / sizeof(ProtocolHandler_t);

